@@ -1,15 +1,15 @@
 /**************************************************************************/
 /*                                                                        */
-/* Copyright (c) 2001, 2006 NoMachine, http://www.nomachine.com/.         */
+/* Copyright (c) 2001, 2007 NoMachine, http://www.nomachine.com/.         */
 /*                                                                        */
-/* NXPROXY, NX protocol compression and NX extensions to this software    */
+/* NXCOMP, NX protocol compression and NX extensions to this software     */
 /* are copyright of NoMachine. Redistribution and use of the present      */
 /* software is allowed according to terms specified in the file LICENSE   */
 /* which comes in the source distribution.                                */
 /*                                                                        */
 /* Check http://www.nomachine.com/licensing.html for applicability.       */
 /*                                                                        */
-/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
+/* NX and NoMachine are trademarks of NoMachine S.r.l.                    */
 /*                                                                        */
 /* All rights reserved.                                                   */
 /*                                                                        */
@@ -32,6 +32,7 @@
 #include "Proxy.h"
 
 #include "Keeper.h"
+#include "Fork.h"
 
 //
 // Set the verbosity level.
@@ -108,27 +109,28 @@ int NXTransDialog(const char *caption, const char *message,
           << getpid() << "'.\n" << logofs_flush;
   #endif
 
-  pid = fork();
+  pid = Fork();
 
   if (pid != 0)
   {
     if (pid < 0)
     {
       #ifdef TEST
-      *logofs << "NXTransDialog: WARNING! Function fork failed "
-              << "with result '" << pid << "'. Error is " << EGET()
-              << " '" << ESTR() << "'.\n" << logofs_flush;
+      *logofs << "NXTransDialog: WARNING! Function fork failed. "
+              << "Error is " << EGET() << " '" << ESTR()
+              << "'.\n" << logofs_flush;
       #endif
 
-      cerr << "Warning" << ": Function fork failed with result '"
-           << pid << "'. Error is " << EGET() << " '" << ESTR()
+      cerr << "Warning" << ": Function fork failed. "
+           << "Error is " << EGET() << " '" << ESTR()
            << "'.\n";
     }
     #ifdef TEST
     else
     {
-      *logofs << "NXTransDialog: Created NX dialog process with pid '"
-              << pid << "'.\n" << logofs_flush;
+      *logofs << "NXTransDialog: Created NX dialog process "
+              << "with pid '" << pid << "'.\n"
+              << logofs_flush;
     }
     #endif
 
@@ -136,9 +138,9 @@ int NXTransDialog(const char *caption, const char *message,
   }
 
   #ifdef TEST
-  *logofs << "NXTransDialog: Executing child with pid '" << getpid()
-          << "' and parent '" << getppid() << "'.\n"
-          << logofs_flush;
+  *logofs << "NXTransDialog: Executing child with pid '"
+          << getpid() << "' and parent '" << getppid()
+          << "'.\n" << logofs_flush;
   #endif
 
   SystemCleanup("NXTransDialog");
@@ -316,27 +318,28 @@ int NXTransClient(const char* display)
           << getpid() << "'.\n" << logofs_flush;
   #endif
 
-  pid = fork();
+  pid = Fork();
 
   if (pid != 0)
   {
     if (pid < 0)
     {
       #ifdef TEST
-      *logofs << "NXTransClient: WARNING! Function fork failed "
-              << "with result '" << pid << "'. Error is " << EGET()
-              << " '" << ESTR() << "'.\n" << logofs_flush;
+      *logofs << "NXTransClient: WARNING! Function fork failed. "
+              << "Error is " << EGET() << " '" << ESTR()
+              << "'.\n" << logofs_flush;
       #endif
 
-      cerr << "Warning" << ": Function fork failed with result '"
-           << pid << "'. Error is " << EGET() << " '" << ESTR()
+      cerr << "Warning" << ": Function fork failed. "
+           << "Error is " << EGET() << " '" << ESTR()
            << "'.\n";
     }
     #ifdef TEST
     else
     {
-      *logofs << "NXTransClient: Created NX client process with pid '"
-              << pid << "'.\n" << logofs_flush;
+      *logofs << "NXTransClient: Created NX client process "
+              << "with pid '" << pid << "'.\n"
+              << logofs_flush;
     }
     #endif
 
@@ -344,9 +347,9 @@ int NXTransClient(const char* display)
   }
 
   #ifdef TEST
-  *logofs << "NXTransClient: Executing child with pid '" << getpid()
-          << "' and parent '" << getppid() << "'.\n"
-          << logofs_flush;
+  *logofs << "NXTransClient: Executing child with pid '"
+          << getpid() << "' and parent '" << getppid()
+          << "'.\n" << logofs_flush;
   #endif
 
   SystemCleanup("NXTransClient");
@@ -510,20 +513,20 @@ int NXTransWatchdog(int timeout)
           << getpid() << "'.\n" << logofs_flush;
   #endif
 
-  pid = fork();
+  pid = Fork();
 
   if (pid != 0)
   {
     if (pid < 0)
     {
       #ifdef TEST
-      *logofs << "NXTransWatchdog: WARNING! Function fork failed "
-              << "with result '" << pid << "'. Error is " << EGET()
-              << " '" << ESTR() << "'.\n" << logofs_flush;
+      *logofs << "NXTransWatchdog: WARNING! Function fork failed. "
+              << "Error is " << EGET() << " '" << ESTR()
+              << "'.\n" << logofs_flush;
       #endif
 
-      cerr << "Warning" << ": Function fork failed with result '"
-           << pid << "'. Error is " << EGET() << " '" << ESTR()
+      cerr << "Warning" << ": Function fork failed. "
+           << "Error is " << EGET() << " '" << ESTR()
            << "'.\n";
     }
     #ifdef TEST
@@ -697,27 +700,28 @@ int NXTransKeeper(int caches, int images, const char *root)
           << getpid() << "'.\n" << logofs_flush;
   #endif
 
-  pid = fork();
+  pid = Fork();
 
   if (pid != 0)
   {
     if (pid < 0)
     {
       #ifdef TEST
-      *logofs << "NXTransKeeper: WARNING! Function fork failed with "
-              << "result '" << pid << "'. Error is " << EGET()
-              << " '" << ESTR() << "'.\n" << logofs_flush;
+      *logofs << "NXTransKeeper: WARNING! Function fork failed. "
+              << "Error is " << EGET() << " '" << ESTR()
+              << "'.\n" << logofs_flush;
       #endif
 
-      cerr << "Warning" << ": Function fork failed with result '"
-           << pid << "'. Error is " << EGET() << " '" << ESTR()
+      cerr << "Warning" << ": Function fork failed. "
+           << "Error is " << EGET() << " '" << ESTR()
            << "'.\n";
     }
     #ifdef TEST
     else
     {
-      *logofs << "NXTransKeeper: Created NX keeper process with pid '"
-              << pid << "'.\n" << logofs_flush;
+      *logofs << "NXTransKeeper: Created NX keeper process "
+              << "with pid '" << pid << "'.\n"
+              << logofs_flush;
     }
     #endif
 
@@ -727,9 +731,9 @@ int NXTransKeeper(int caches, int images, const char *root)
   int parent = getppid();
 
   #ifdef TEST
-  *logofs << "NXTransKeeper: Executing child with pid '" << getpid()
-          << "' and parent '" << parent << "'.\n"
-          << logofs_flush;
+  *logofs << "NXTransKeeper: Executing child with pid '"
+          << getpid() << "' and parent '" << parent
+          << "'.\n" << logofs_flush;
   #endif
 
   SystemCleanup("NXTransKeeper");
@@ -797,18 +801,16 @@ int NXTransKeeper(int caches, int images, const char *root)
   }
 
   //
-  // Delay the first run to give a boost
-  // to session startup.
+  // Delay a bit the first run to give
+  // a boost to the session startup.
   //
 
-  NXTransKeeperCheck();
-
   #ifdef TEST
-  *logofs << "NXTransKeeper: Going to sleep for " << timeout
-          << " Ms.\n" << logofs_flush;
+  *logofs << "NXTransKeeper: Going to sleep for "
+          << timeout / 20 << " Ms.\n" << logofs_flush;
   #endif
 
-  usleep(timeout * 1000);
+  usleep(timeout / 20 * 1000);
 
   NXTransKeeperCheck();
 
