@@ -1,15 +1,15 @@
 /**************************************************************************/
 /*                                                                        */
-/* Copyright (c) 2001, 2006 NoMachine, http://www.nomachine.com/.         */
+/* Copyright (c) 2001, 2007 NoMachine, http://www.nomachine.com/.         */
 /*                                                                        */
-/* NXPROXY, NX protocol compression and NX extensions to this software    */
+/* NXCOMP, NX protocol compression and NX extensions to this software     */
 /* are copyright of NoMachine. Redistribution and use of the present      */
 /* software is allowed according to terms specified in the file LICENSE   */
 /* which comes in the source distribution.                                */
 /*                                                                        */
 /* Check http://www.nomachine.com/licensing.html for applicability.       */
 /*                                                                        */
-/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
+/* NX and NoMachine are trademarks of NoMachine S.r.l.                    */
 /*                                                                        */
 /* All rights reserved.                                                   */
 /*                                                                        */
@@ -34,7 +34,7 @@
 // Maximum number of bytes sent for each token.
 //
 
-#define TOKEN_SIZE                               4096
+#define TOKEN_SIZE                               1536
 
 //
 // Maximum number of tokens that can be spent
@@ -44,13 +44,6 @@
 
 #define TOKEN_LIMIT                              24
 
-//
-// Report a congestion if tokens come below this
-// threshold.
-//
-
-#define TOKEN_THRESHOLD                          12
-
 //
 // By default assume the proxy is running as a
 // standalone program.
@@ -81,26 +74,6 @@
 
 #define INIT_TIMEOUT                             60000
 
-//
-// How many milliseconds to wait before to restart
-// handling of pending messages after a timeout.
-//
-
-#define PENDING_TIMEOUT                          0
-
-//
-// Only send one motion event any N milliseconds.
-//
-
-#define MOTION_TIMEOUT                           0
-
-//
-// Send a ping to remote proxy if no other data
-// was sent for more than this amount of time.
-//
-
-#define FRAME_TIMEOUT                            50
-
 //
 // Enter the congestion state if the remote does
 // not reply to a ping within the given amount
@@ -110,14 +83,18 @@
 #define PING_TIMEOUT                             5000
 
 //
-// At connection reset, server channel needs to
-// sinchronize its sequence counter with sequence
-// of next event or reply coming from the X server.
-// To do that, it will be able to wait for this
-// amount of milliseconds.
+// Only send one motion event any N milliseconds.
 //
 
-#define RESET_TIMEOUT                            5000
+#define MOTION_TIMEOUT                           0
+
+
+//
+// Force an update of the congestion counter if
+// the proxy is idle for this time.
+//
+
+#define IDLE_TIMEOUT                             50
 
 //
 // Close X connection if can't write before this
@@ -133,27 +110,12 @@
 
 #define PROXY_TIMEOUT                            120000
 
-//
-// Timeout used when waiting for a new complete
-// message to become available.
-//
-
-#define RETRY_TIMEOUT                            500
-
 //
 // How many milliseconds to wait for the shared
 // memory completion event to become available.
 //
 
-#define SHMEM_TIMEOUT                            0
-
-//
-// Time before we consider concluded the statup
-// of X applications making the user's session.
-//
-
-#define STARTUP_TIMEOUT                          10000
-
+#define SHMEM_TIMEOUT                            200
 //
 // Before closing down the proxy, wait for the
 // given amount of miliseconds to let all the
@@ -172,14 +134,7 @@
 #define CLEANUP_TIMEOUT                          3000
 
 //
-// Slow down the client proxy if X server can't
-// keep up handling X messages.
-//
-
-#define CONGESTION_TIMEOUT                       0
-
-//
-// Wait this amount of milliseconds before any
+// Wait this amount of milliseconds after any
 // iteration of the house-keeping process.
 //
 
@@ -199,51 +154,27 @@
 // and other classes.
 //
 
-#define TRANSPORT_X_BUFFER_SIZE                  32768
-#define TRANSPORT_PROXY_BUFFER_SIZE              32768
-#define TRANSPORT_GENERIC_BUFFER_SIZE            65536
+#define TRANSPORT_X_BUFFER_SIZE                  131072
+#define TRANSPORT_PROXY_BUFFER_SIZE              65536
+#define TRANSPORT_GENERIC_BUFFER_SIZE            16384
 
-#define TRANSPORT_X_BUFFER_LIMIT                 524288
-#define TRANSPORT_PROXY_BUFFER_LIMIT             327680
-#define TRANSPORT_GENERIC_BUFFER_LIMIT           131072
+#define TRANSPORT_X_BUFFER_THRESHOLD             262144
+#define TRANSPORT_PROXY_BUFFER_THRESHOLD         131072
+#define TRANSPORT_GENERIC_BUFFER_THRESHOLD       32768
 
 //
-// Stop doubling the size and print a warning
-// if a buffer reaches this limit.
+// Never allow buffers to exceed this limit.
 //
 
-#define TRANSPORT_MAXIMUM_BUFFER_SIZE            524288
+#define TRANSPORT_MAXIMUM_BUFFER_SIZE            393216
 
 //
-// Never allow X buffers to exceed this limit.
+// Immediately flush the accumulated data to
+// the X server if the write buffer exceeds
+// this size.
 //
 
-#define TRANSPORT_FORCE_BUFFER_SIZE              65536
-
-//
-// Stop reading from proxy or X connections
-// if the data stored in buffer exceed these
-// limits.
-//
-
-#define TRANSPORT_X_BUFFER_THRESHOLD             327680
-#define TRANSPORT_PROXY_BUFFER_THRESHOLD         32768
-#define TRANSPORT_GENERIC_BUFFER_THRESHOLD       65536
-
-//
-// Immediately flush data to the X socket
-// if buffer exceeds this limit.
-//
-
-#define TRANSPORT_WRITE_THRESHOLD                4096
-
-//
-// Write only this amount of bytes on sockets
-// at each write. Write the rest when socket
-// is reported to be writable.
-//
-
-#define TRANSPORT_WRITE_LIMIT                    1048576
+#define TRANSPORT_FLUSH_BUFFER_SIZE              16384
 
 //
 // Defaults used for socket options.
@@ -288,8 +219,8 @@
 // Defaults used for image cache.
 //
 
-#define IMAGE_CACHE_ENABLE_LOAD                  1
-#define IMAGE_CACHE_ENABLE_SAVE                  1
+#define IMAGE_CACHE_ENABLE_LOAD                  0
+#define IMAGE_CACHE_ENABLE_SAVE                  0
 
 #define IMAGE_CACHE_DISK_LIMIT                   33554432
 
@@ -298,17 +229,17 @@
 // used by read buffer classes.
 //
 
-#define CLIENT_INITIAL_READ_SIZE                 4096
-#define CLIENT_MAXIMUM_READ_SIZE                 32768
+#define CLIENT_INITIAL_READ_SIZE                 8192
+#define CLIENT_MAXIMUM_BUFFER_SIZE               262144
 
-#define SERVER_INITIAL_READ_SIZE                 1024
-#define SERVER_MAXIMUM_READ_SIZE                 16384
+#define SERVER_INITIAL_READ_SIZE                 8192
+#define SERVER_MAXIMUM_BUFFER_SIZE               65536
 
-#define PROXY_INITIAL_READ_SIZE                  8192
-#define PROXY_MAXIMUM_READ_SIZE                  16384
+#define PROXY_INITIAL_READ_SIZE                  65536
+#define PROXY_MAXIMUM_BUFFER_SIZE                262144 + 1024
 
-#define GENERIC_INITIAL_READ_SIZE                1024
-#define GENERIC_MAXIMUM_READ_SIZE                1024
+#define GENERIC_INITIAL_READ_SIZE                8192
+#define GENERIC_MAXIMUM_BUFFER_SIZE              8192
 
 //
 // Calculate bitrate in given time frames.
@@ -327,16 +258,6 @@
 #define CLIENT_BITRATE_LIMIT                     0
 #define SERVER_BITRATE_LIMIT                     0
 
-//
-// CPU limit. Value is ratio between time spent
-// in select and time spent handling messages.
-// A value of 0 means no limit. By default set
-// a limit only at X client side.
-//
-
-#define CLIENT_PROCESSOR_LIMIT                   0
-#define SERVER_PROCESSOR_LIMIT                   0
-
 //
 // Default values for cache control. We limit
 // the maximum size of a request to 262144 but
@@ -416,13 +337,8 @@
 //
 // Reopen the log file if it exceeds this size.
 //
-/*
-FIXME: This is a temporary while investigating the problems
-       with the splits.
 
 #define FILE_SIZE_LIMIT                          60000000
-*/
-#define FILE_SIZE_LIMIT                          2000000000
 
 //
 // Check periodically if we need to truncate the
@@ -447,25 +363,20 @@ Control::Control()
   LinkEncrypted = LINK_ENCRYPTED;
   FlushPriority = FLUSH_PRIORITY;
 
-  TokenSize      = TOKEN_SIZE;
-  TokenLimit     = TOKEN_LIMIT;
-  TokenThreshold = TOKEN_THRESHOLD;
+  TokenSize  = TOKEN_SIZE;
+  TokenLimit = TOKEN_LIMIT;
 
   ChannelMask = CHANNEL_MASK;
 
-  InitTimeout    = INIT_TIMEOUT;
-  MotionTimeout  = MOTION_TIMEOUT;
-  PendingTimeout = PENDING_TIMEOUT;
-  FrameTimeout   = FRAME_TIMEOUT;
-  PingTimeout    = PING_TIMEOUT;
+  InitTimeout   = INIT_TIMEOUT;
+  PingTimeout   = PING_TIMEOUT;
+  MotionTimeout = MOTION_TIMEOUT;
+  IdleTimeout   = IDLE_TIMEOUT;
 
-  ResetTimeout   = RESET_TIMEOUT;
   ChannelTimeout = CHANNEL_TIMEOUT;
   ProxyTimeout   = PROXY_TIMEOUT;
-  RetryTimeout   = RETRY_TIMEOUT;
   ShmemTimeout   = SHMEM_TIMEOUT;
 
-  StartupTimeout  = STARTUP_TIMEOUT;
   CleanupTimeout  = CLEANUP_TIMEOUT;
   KeeperTimeout   = KEEPER_TIMEOUT;
   LatencyTimeout  = LATENCY_TIMEOUT;
@@ -528,23 +439,18 @@ Control::Control()
   TransportProxyBufferSize        = TRANSPORT_PROXY_BUFFER_SIZE;
   TransportGenericBufferSize      = TRANSPORT_GENERIC_BUFFER_SIZE;
 
-  TransportXBufferLimit           = TRANSPORT_X_BUFFER_LIMIT;
-  TransportProxyBufferLimit       = TRANSPORT_PROXY_BUFFER_LIMIT;
-  TransportGenericBufferLimit     = TRANSPORT_GENERIC_BUFFER_LIMIT;
-
-  TransportMaximumBufferSize      = TRANSPORT_MAXIMUM_BUFFER_SIZE;
-  TransportForceBufferSize        = TRANSPORT_FORCE_BUFFER_SIZE;
-
   TransportXBufferThreshold       = TRANSPORT_X_BUFFER_THRESHOLD;
   TransportProxyBufferThreshold   = TRANSPORT_PROXY_BUFFER_THRESHOLD;
   TransportGenericBufferThreshold = TRANSPORT_GENERIC_BUFFER_THRESHOLD;
 
+  TransportMaximumBufferSize      = TRANSPORT_MAXIMUM_BUFFER_SIZE;
+
   //
-  // Set write policies to X connections.
+  // Flush the write buffer if it exceeds
+  // this size.
   //
 
-  TransportWriteLimit     = TRANSPORT_WRITE_LIMIT;
-  TransportWriteThreshold = TRANSPORT_WRITE_THRESHOLD;
+  TransportFlushBufferSize = TRANSPORT_FLUSH_BUFFER_SIZE;
 
   //
   // Socket options.
@@ -610,20 +516,20 @@ Control::Control()
   ImageCacheDiskLimit = IMAGE_CACHE_DISK_LIMIT;
 
   //
-  // Size of read operations in read buffer classes.
+  // Set defaults for the read buffers.
   //
 
-  ClientInitialReadSize  = CLIENT_INITIAL_READ_SIZE;
-  ClientMaximumReadSize  = CLIENT_MAXIMUM_READ_SIZE;
+  ClientInitialReadSize    = CLIENT_INITIAL_READ_SIZE;
+  ClientMaximumBufferSize  = CLIENT_MAXIMUM_BUFFER_SIZE;
 
-  ServerInitialReadSize  = SERVER_INITIAL_READ_SIZE;
-  ServerMaximumReadSize  = SERVER_MAXIMUM_READ_SIZE;
+  ServerInitialReadSize    = SERVER_INITIAL_READ_SIZE;
+  ServerMaximumBufferSize  = SERVER_MAXIMUM_BUFFER_SIZE;
 
-  ProxyInitialReadSize   = PROXY_INITIAL_READ_SIZE;
-  ProxyMaximumReadSize   = PROXY_MAXIMUM_READ_SIZE;
+  ProxyInitialReadSize     = PROXY_INITIAL_READ_SIZE;
+  ProxyMaximumBufferSize   = PROXY_MAXIMUM_BUFFER_SIZE;
 
-  GenericInitialReadSize = GENERIC_INITIAL_READ_SIZE;
-  GenericMaximumReadSize = GENERIC_MAXIMUM_READ_SIZE;
+  GenericInitialReadSize   = GENERIC_INITIAL_READ_SIZE;
+  GenericMaximumBufferSize = GENERIC_MAXIMUM_BUFFER_SIZE;
 
   ShortBitrateTimeFrame = SHORT_BITRATE_TIME_FRAME;
   LongBitrateTimeFrame  = LONG_BITRATE_TIME_FRAME;
@@ -633,14 +539,10 @@ Control::Control()
   //
 
   LocalBitrateLimit   = -1;
-  LocalProcessorLimit = -1;
 
   ClientBitrateLimit = CLIENT_BITRATE_LIMIT;
   ServerBitrateLimit = SERVER_BITRATE_LIMIT;
 
-  ClientProcessorLimit = CLIENT_PROCESSOR_LIMIT;
-  ServerProcessorLimit = SERVER_PROCESSOR_LIMIT;
-
   //
   // Default parameters for message handling.
   //
@@ -753,51 +655,14 @@ Control::Control()
   setLocalUnpackMethods();
 
   //
-  // Set initial value of flow control.
-  //
-
-  bytesIn_  = 0;
-  bytesOut_ = 0;
-
-  idleTime_  = 0;
-  readTime_  = 0;
-  writeTime_ = 0;
-
-  //
-  // Track the bitrate of I/O operations
-  // on the proxy link.
-  //
-
-  startShortFrameTs_ = getTimestamp();
-  startLongFrameTs_  = getTimestamp();
-  startFrameTs_      = getTimestamp();
-
-  bytesInShortFrame_ = 0;
-  bytesInLongFrame_  = 0;
-
-  bitrateInShortFrame_ = 0;
-  bitrateInLongFrame_  = 0;
-
-  topBitrate_ = 0;
-
-  //
-  // Track startup time.
-  //
-
-  startupTs_ = getTimestamp();
-
-  isStartup_ = 1;
-
-  //
-  // Will have to set the protocol
-  // version at the time session
-  // is negotiated.
+  // Set the protocol version at the
+  // time the session is negotiated.
   //
 
   protoStep6_ = 0;
   protoStep7_ = 0;
-
-  cacheStep3_ = 0;
+  protoStep8_ = 0;
+  protoStep9_ = 0;
 }
 
 Control::~Control()
@@ -859,131 +724,90 @@ Control::~Control()
 }
 
 //
-// Recalculate the current bitrate. The bytes written
-// are accounted at the time the transport actually
-// writes the data to the network, not at the time it
-// receives the data from the upper layers. The reason
-// is that data can be compressed by the stream com-
-// pressor, so we can become aware of the new bitrate
-// only afer having flushed the ZLIB stream. This also
-// means that, to have a reliable estimate, we need to
-// flush the link often.
+// Set the protocol step based on the
+// remote version.
 //
 
-void Control::updateBitrate(int bytes)
+void Control::setProtoStep(int step)
 {
-  T_timestamp thisFrameTs = getNewTimestamp();
-
-  int diffFramesInMs = diffTimestamp(startFrameTs_, thisFrameTs);
-
-  #ifdef DEBUG
-  *logofs << "Control: Difference since previous timestamp is "
-          << diffFramesInMs << " Ms.\n" << logofs_flush;
-  #endif
-
-  if (diffFramesInMs > 0)
+  switch (step)
   {
-    #ifdef DEBUG
-    *logofs << "Control: Removing " << diffFramesInMs
-            << " Ms in short and long time frame.\n"
-            << logofs_flush;
-    #endif
-
-    int shortBytesToRemove = (int) (((double) bytesInShortFrame_ * (double) diffFramesInMs) /
-                                       (double) ShortBitrateTimeFrame);
-
-    int longBytesToRemove = (int) (((double) bytesInLongFrame_ * (double) diffFramesInMs) /
-                                      (double) LongBitrateTimeFrame);
-
-    #ifdef DEBUG
-    *logofs << "Control: Removing " << shortBytesToRemove
-            << " bytes from " << bytesInShortFrame_
-            << " in the short frame.\n" << logofs_flush;
-    #endif
-
-    bytesInShortFrame_ -= shortBytesToRemove;
-
-    if (bytesInShortFrame_ < 0)
+    case 6:
     {
-      #ifdef TEST
-      *logofs << "Control: Bytes in short frame are "
-              << bytesInShortFrame_ << ". Set to 0.\n"
+      protoStep6_ = 1;
+      protoStep7_ = 0;
+      protoStep8_ = 0;
+      protoStep9_ = 0;
+
+      break;
+    }
+    case 7:
+    {
+      protoStep6_ = 1;
+      protoStep7_ = 1;
+      protoStep8_ = 0;
+      protoStep9_ = 0;
+
+      break;
+    }
+    case 8:
+    {
+      protoStep6_ = 1;
+      protoStep7_ = 1;
+      protoStep8_ = 1;
+      protoStep9_ = 0;
+
+      break;
+    }
+    case 9:
+    {
+      protoStep6_ = 1;
+      protoStep7_ = 1;
+      protoStep8_ = 1;
+      protoStep9_ = 1;
+
+      break;
+    }
+    default:
+    {
+      #ifdef PANIC
+      *logofs << "Control: PANIC! Invalid protocol step "
+              << "with value " << step << ".\n"
               << logofs_flush;
       #endif
 
-      bytesInShortFrame_ = 0;
+      HandleCleanup();
     }
+  }
+}
 
-    #ifdef DEBUG
-    *logofs << "Control: Removing " << longBytesToRemove
-            << " bytes from " << bytesInLongFrame_
-            << " in the long frame.\n" << logofs_flush;
+int Control::getProtoStep()
+{
+  if (protoStep9_ == 1)
+  {
+    return 9;
+  }
+  else if (protoStep8_ == 1)
+  {
+    return 8;
+  }
+  else if (protoStep7_ == 1)
+  {
+    return 7;
+  }
+  else if (protoStep6_ == 1)
+  {
+    return 6;
+  }
+  else
+  {
+    #ifdef PANIC
+    *logofs << "Control: PANIC! Can't identify the "
+            << "protocol step.\n" << logofs_flush;
     #endif
 
-    bytesInLongFrame_ -= longBytesToRemove;
-
-    if (bytesInLongFrame_ < 0)
-    {
-      #ifdef TEST
-      *logofs << "Control: Bytes in long frame are "
-              << bytesInLongFrame_ << ". Set to 0.\n"
-              << logofs_flush;
-      #endif
-
-      bytesInLongFrame_ = 0;
-    }
-
-    int diffStartInMs;
-
-    diffStartInMs = diffTimestamp(thisFrameTs, startShortFrameTs_);
-
-    if (diffStartInMs > ShortBitrateTimeFrame)
-    {
-      addMsToTimestamp(startShortFrameTs_, diffStartInMs);
-    }
-
-    diffStartInMs = diffTimestamp(thisFrameTs, startLongFrameTs_);
-
-    if (diffStartInMs > LongBitrateTimeFrame)
-    {
-      addMsToTimestamp(startLongFrameTs_, diffStartInMs);
-    }
-
-    startFrameTs_ = thisFrameTs;
+    HandleCleanup();
   }
-
-  #ifdef DEBUG
-  *logofs << "Control: Adding " << bytes << " bytes to "
-          << bytesInShortFrame_ << " in the short frame.\n"
-          << logofs_flush;
-  #endif
-
-  bytesInShortFrame_ = bytesInShortFrame_ + bytes;
-
-  #ifdef DEBUG
-  *logofs << "Control: Adding " << bytes << " bytes to "
-          << bytesInLongFrame_ << " in the long frame.\n"
-          << logofs_flush;
-  #endif
-
-  bytesInLongFrame_ = bytesInLongFrame_ + bytes;
-
-  bitrateInShortFrame_ = (int) ((double) bytesInShortFrame_ /
-                                   ((double) ShortBitrateTimeFrame / 1000));
-
-  bitrateInLongFrame_ = (int) ((double) bytesInLongFrame_ /
-                                   ((double) LongBitrateTimeFrame / 1000));
-
-  if (bitrateInShortFrame_ > topBitrate_)
-  {
-    topBitrate_ = bitrateInShortFrame_;
-  }
-
-  #ifdef TEST
-  *logofs << "Control: Current bitrate is short " << bitrateInShortFrame_
-          << " long " << bitrateInLongFrame_ << " top " << topBitrate_
-          << ".\n" << logofs_flush;
-  #endif
 }
 
 //
@@ -1021,33 +845,6 @@ void Control::setLocalUnpackMethods()
 
   LocalUnpackMethods[PACK_COLORMAP_256_COLORS]        = 1;
 
-  LocalUnpackMethods[PACK_RDP_PLAIN_256_COLORS]       = 1;
-  LocalUnpackMethods[PACK_RDP_COMPRESSED_256_COLORS]  = 1;
-  LocalUnpackMethods[PACK_RDP_PLAIN_32K_COLORS]       = 1;
-  LocalUnpackMethods[PACK_RDP_COMPRESSED_32K_COLORS]  = 1;
-  LocalUnpackMethods[PACK_RDP_PLAIN_64K_COLORS]       = 1;
-  LocalUnpackMethods[PACK_RDP_COMPRESSED_64K_COLORS]  = 1;
-  LocalUnpackMethods[PACK_RDP_PLAIN_16M_COLORS]       = 1;
-  LocalUnpackMethods[PACK_RDP_COMPRESSED_16M_COLORS]  = 1;
-
-  LocalUnpackMethods[PACK_RDP_TEXT]                   = 1;
-
-  LocalUnpackMethods[PACK_RFB_HEXTILE]                = 1;
-
-  LocalUnpackMethods[PACK_RFB_TIGHT_PLAIN]            = 1;
-  LocalUnpackMethods[PACK_RFB_TIGHT_COMPRESSED]       = 1;
-
-  LocalUnpackMethods[PACK_TIGHT_8_COLORS]             = 1;
-  LocalUnpackMethods[PACK_TIGHT_64_COLORS]            = 1;
-  LocalUnpackMethods[PACK_TIGHT_256_COLORS]           = 1;
-  LocalUnpackMethods[PACK_TIGHT_512_COLORS]           = 1;
-  LocalUnpackMethods[PACK_TIGHT_4K_COLORS]            = 1;
-  LocalUnpackMethods[PACK_TIGHT_32K_COLORS]           = 1;
-  LocalUnpackMethods[PACK_TIGHT_64K_COLORS]           = 1;
-  LocalUnpackMethods[PACK_TIGHT_256K_COLORS]          = 1;
-  LocalUnpackMethods[PACK_TIGHT_2M_COLORS]            = 1;
-  LocalUnpackMethods[PACK_TIGHT_16M_COLORS]           = 1;
-
   LocalUnpackMethods[PACK_JPEG_8_COLORS]              = 1;
   LocalUnpackMethods[PACK_JPEG_64_COLORS]             = 1;
   LocalUnpackMethods[PACK_JPEG_256_COLORS]            = 1;
@@ -1070,23 +867,11 @@ void Control::setLocalUnpackMethods()
   LocalUnpackMethods[PACK_PNG_2M_COLORS]              = 1;
   LocalUnpackMethods[PACK_PNG_16M_COLORS]             = 1;
 
-  LocalUnpackMethods[PACK_PNG_JPEG_8_COLORS]          = 1;
-  LocalUnpackMethods[PACK_PNG_JPEG_64_COLORS]         = 1;
-  LocalUnpackMethods[PACK_PNG_JPEG_256_COLORS]        = 1;
-  LocalUnpackMethods[PACK_PNG_JPEG_512_COLORS]        = 1;
-  LocalUnpackMethods[PACK_PNG_JPEG_4K_COLORS]         = 1;
-  LocalUnpackMethods[PACK_PNG_JPEG_32K_COLORS]        = 1;
-  LocalUnpackMethods[PACK_PNG_JPEG_64K_COLORS]        = 1;
-  LocalUnpackMethods[PACK_PNG_JPEG_256K_COLORS]       = 1;
-  LocalUnpackMethods[PACK_PNG_JPEG_2M_COLORS]         = 1;
-  LocalUnpackMethods[PACK_PNG_JPEG_16M_COLORS]        = 1;
-
   LocalUnpackMethods[PACK_RGB_16M_COLORS]             = 1;
-/*
-FIXME: The RLE pack method must be implemented.
-*/
   LocalUnpackMethods[PACK_RLE_16M_COLORS]             = 1;
 
   LocalUnpackMethods[PACK_ALPHA]                      = 1;
   LocalUnpackMethods[PACK_COLORMAP]                   = 1;
+
+  LocalUnpackMethods[PACK_BITMAP_16M_COLORS]          = 1;
 }
