@@ -1,15 +1,15 @@
 /**************************************************************************/
 /*                                                                        */
-/* Copyright (c) 2001, 2006 NoMachine, http://www.nomachine.com/.         */
+/* Copyright (c) 2001, 2007 NoMachine, http://www.nomachine.com/.         */
 /*                                                                        */
-/* NXPROXY, NX protocol compression and NX extensions to this software    */
+/* NXCOMP, NX protocol compression and NX extensions to this software     */
 /* are copyright of NoMachine. Redistribution and use of the present      */
 /* software is allowed according to terms specified in the file LICENSE   */
 /* which comes in the source distribution.                                */
 /*                                                                        */
 /* Check http://www.nomachine.com/licensing.html for applicability.       */
 /*                                                                        */
-/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
+/* NX and NoMachine are trademarks of NoMachine S.r.l.                    */
 /*                                                                        */
 /* All rights reserved.                                                   */
 /*                                                                        */
@@ -62,22 +62,23 @@ typedef enum
   stage_sending_proxy_options,
   stage_waiting_proxy_caches,
   stage_sending_proxy_caches,
-  stage_operational
+  stage_operational,
+  stage_terminating,
+  stage_terminated
 }
 T_proxy_stage;
 
 //
-// This is a suggestion about the type
-// of session that's running.
+// Hint about whether or not the proxy is
+// connected to a NX agen.
 //
 
 typedef enum
 {
   session_undefined = -1,
-  session_x,
-  session_rdp,
-  session_rfb,
-  session_raw,
+  session_agent,
+  session_shadow,
+  session_proxy,
   session_last_tag
 }
 T_session_mode;
@@ -180,13 +181,6 @@ class Control
 
   int TokenLimit;
 
-  //
-  // Report a congestion if the tokens come below
-  // this threshold.
-  //
-
-  int TokenThreshold;
-
   //
   // Bitmask used to determine the distribution
   // of channel ids between the client and server
@@ -202,27 +196,6 @@ class Control
 
   int InitTimeout;
 
-  //
-  // How many milliseconds to wait before to restart
-  // handling of pending messages after a timeout.
-  //
-
-  int PendingTimeout;
-
-  //
-  // Enqueue motion notify events in server channel.
-  //
-
-  int MotionTimeout;
-
-  //
-  // Agents can try to keep the frequency of packets
-  // constant by sending a null request if no other
-  // data was produced in the given timeframe.
-  //
-
-  int FrameTimeout;
-
   //
   // Enter the congestion state if the remote does
   // not reply to the ping within the given amount
@@ -232,10 +205,17 @@ class Control
   int PingTimeout;
 
   //
-  // Timeout used by server channel at reset.
+  // Enqueue motion notify events in server channel.
   //
 
-  int ResetTimeout;
+  int MotionTimeout;
+
+  //
+  // Force an update of the congestion counter if
+  // the proxy is idle for this time.
+  //
+
+  int IdleTimeout;
 
   //
   // Close the connection if can't write before
@@ -251,13 +231,6 @@ class Control
 
   int ProxyTimeout;
 
-  //
-  // Timeout used to wait for a complete message
-  // to become available.
-  //
-
-  int RetryTimeout;
-
   //
   // How many milliseconds to wait for the shared
   // memory completion event to become available.
@@ -265,16 +238,6 @@ class Control
 
   int ShmemTimeout;
 
-  //
-  // How many milliseconds since we connected to the
-  // remote proxy is to be considered startup time of
-  // the session? All messages that are stored in
-  // cache during this period receive an adjuntive
-  // hit bonus.
-  //
-
-  int StartupTimeout;
-
   //
   // Wait for applications to complete at the time
   // proxy is shut down.
@@ -402,16 +365,16 @@ class Control
   //
 
   int ClientInitialReadSize;
-  int ClientMaximumReadSize;
+  int ClientMaximumBufferSize;
 
   int ServerInitialReadSize;
-  int ServerMaximumReadSize;
+  int ServerMaximumBufferSize;
 
   int ProxyInitialReadSize;
-  int ProxyMaximumReadSize;
+  int ProxyMaximumBufferSize;
 
   int GenericInitialReadSize;
-  int GenericMaximumReadSize;
+  int GenericMaximumBufferSize;
 
   //
   // Set initial size and resize policy of
@@ -423,35 +386,18 @@ class Control
   int TransportProxyBufferSize;
   int TransportGenericBufferSize;
 
-  int TransportXBufferLimit;
-  int TransportProxyBufferLimit;
-  int TransportGenericBufferLimit;
-
-  int TransportMaximumBufferSize;
-  int TransportForceBufferSize;
-
-  //
-  // Stop reading from proxy if channels
-  // exceed these buffer limits.
-  //
-
   int TransportXBufferThreshold;
   int TransportProxyBufferThreshold;
   int TransportGenericBufferThreshold;
 
-  //
-  // Immediately flush data produced for the
-  // X connection if it exceeds this limit.
-  //
-
-  int TransportWriteThreshold;
+  int TransportMaximumBufferSize;
 
   //
-  // Set this to the maximum amount of data that 
-  // will be sent in a single write operation.
+  // Flush the data produced for the channel
+  // connection if it exceeds this size.
   //
 
-  int TransportWriteLimit;
+  int TransportFlushBufferSize;
 
   //
   // Socket options.
@@ -486,19 +432,15 @@ class Control
   int LongBitrateTimeFrame;
 
   //
-  // Limit bandwidth on proxy link and
-  // CPU time spent handling messages.
+  // Limit the bandwidth usage of the proxy
+  // link.
   //
 
   int LocalBitrateLimit;
-  int LocalProcessorLimit;
 
   int ClientBitrateLimit;
   int ServerBitrateLimit;
   
-  int ClientProcessorLimit;
-  int ServerProcessorLimit;
-  
   //
   // This is the limit imposed by user on
   // total cache size.
@@ -758,167 +700,23 @@ class Control
                         ServerTotalStorageSize);
   }
 
-  //
-  // Track CPU usage.
-  //
+  void setProtoStep(int step);
 
-  void addIdleTime(unsigned int numMs)
-  {
-    idleTime_ += numMs;
-  }
-
-  void subIdleTime(unsigned int numMs)
-  {
-    idleTime_ -= numMs;
-  }
-
-  double getIdleTime() const
-  {
-    return idleTime_;
-  }
-
-  void addReadTime(unsigned int numMs)
-  {
-    readTime_ += numMs;
-  }
-
-  void subReadTime(unsigned int numMs)
-  {
-    readTime_ -= numMs;
-  }
-
-  double getReadTime() const
-  {
-    return readTime_;
-  }
-
-  void addWriteTime(unsigned int numMs)
-  {
-    writeTime_ += numMs;
-  }
-
-  void subWriteTime(unsigned int numMs)
-  {
-    writeTime_ -= numMs;
-  }
-
-  double getWriteTime() const
-  {
-    return writeTime_;
-  }
-
-  //
-  // Track network usage.
-  //
-
-  void addBytesIn(unsigned int numBytes)
-  {
-    bytesIn_ += (double) numBytes;
-  }
-
-  double getBytesIn()
-  {
-    return bytesIn_;
-  }
-
-  void addBytesOut(unsigned int numBytes)
-  {
-    bytesOut_ += (double) numBytes;
-  }
-
-  double getBytesOut()
-  {
-    return bytesOut_;
-  }
-
-  //
-  // Update the current bitrate.
-  //
-
-  void updateBitrate(int bytes);
-
-  //
-  // Return the current bitrate.
-  //
-
-  int getBitrateInShortFrame()
-  {
-    return bitrateInShortFrame_;
-  }
-
-  int getBitrateInLongFrame()
-  {
-    return bitrateInLongFrame_;
-  }
-
-  int getTopBitrate()
-  {
-    return topBitrate_;
-  }
-
-  void resetTopBitrate()
-  {
-    topBitrate_ = 0;
-  }
-
-  double getStreamRatio()
-  {
-    return statistics -> getStreamRatio();
-  }
-
-  int isStartup()
-  {
-    if (isStartup_ == 0)
-    {
-      return 0;
-    }
-    else
-    {
-      if (StartupTimeout >= LatencyTimeout)
-      {
-        isStartup_ = ((StartupTimeout - LatencyTimeout -
-                          diffTimestamp(startupTs_,
-                              getTimestamp())) > 0);
-      }
-      else
-      {
-        isStartup_ = 0;
-      }
-    }
-
-    return isStartup_;
-  }
-
-  void setProtoStep6()
-  {
-    protoStep6_ = 1;
-    protoStep7_ = 0;
-  }
-
-  void setProtoStep7()
-  {
-    protoStep6_ = 1;
-    protoStep7_ = 1;
-  }
-
-  int isProtoStep6()
-  {
-    return protoStep6_;
-  }
+  int getProtoStep();
 
   int isProtoStep7()
   {
     return protoStep7_;
   }
 
-  void setCacheStep3()
+  int isProtoStep8()
   {
-    cacheStep3_ = 1;
+    return protoStep8_;
   }
 
-  int isCacheStep3()
+  int isProtoStep9()
   {
-    return cacheStep3_;
+    return protoStep9_;
   }
 
   private:
@@ -930,53 +728,14 @@ class Control
   void setLocalUnpackMethods();
 
   //
-  // Track CPU and network usage.
-  //
-
-  double idleTime_;
-  double readTime_;
-  double writeTime_;
-
-  double bytesIn_;
-  double bytesOut_;
-
-  //
-  // Track the current bitrate used for
-  // I/O operations on the proxy link.
-  //
-
-  T_timestamp startShortFrameTs_;
-  T_timestamp startLongFrameTs_;
-  T_timestamp startFrameTs_;
-
-  int bytesInShortFrame_;
-  int bytesInLongFrame_;
-
-  int bitrateInShortFrame_;
-  int bitrateInLongFrame_;
-
-  int topBitrate_;
-
-  //
-  // Time since we started the proxy.
-  //
-
-  T_timestamp startupTs_;
-  int isStartup_;
-
-  //
-  // Manage encoding according to
-  // protocol versions.
+  // Manage the encoding according
+  // to the protocol version.
   //
 
   int protoStep6_;
   int protoStep7_;
-
-  //
-  // Manage the version of cache.
-  //
-
-  int cacheStep3_;
+  int protoStep8_;
+  int protoStep9_;
 };
 
 #endif /* Control_H */
