@@ -1,15 +1,15 @@
 /**************************************************************************/
 /*                                                                        */
-/* Copyright (c) 2001, 2006 NoMachine, http://www.nomachine.com/.         */
+/* Copyright (c) 2001, 2007 NoMachine, http://www.nomachine.com/.         */
 /*                                                                        */
-/* NXPROXY, NX protocol compression and NX extensions to this software    */
+/* NXCOMP, NX protocol compression and NX extensions to this software     */
 /* are copyright of NoMachine. Redistribution and use of the present      */
 /* software is allowed according to terms specified in the file LICENSE   */
 /* which comes in the source distribution.                                */
 /*                                                                        */
 /* Check http://www.nomachine.com/licensing.html for applicability.       */
 /*                                                                        */
-/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
+/* NX and NoMachine are trademarks of NoMachine S.r.l.                    */
 /*                                                                        */
 /* All rights reserved.                                                   */
 /*                                                                        */
@@ -29,8 +29,7 @@ class GenericReadBuffer : public ReadBuffer
 
   GenericReadBuffer(Transport *transport, GenericChannel *channel)
 
-    : ReadBuffer(transport, control -> GenericInitialReadSize,
-                     control -> GenericMaximumReadSize), channel_(channel)
+    : ReadBuffer(transport), channel_(channel)
   {
   }
 
@@ -38,13 +37,10 @@ class GenericReadBuffer : public ReadBuffer
   {
   }
 
-  virtual int checkMessage()
-  {
-    return (length_ > 0);
-  }
-
   protected:
 
+  virtual unsigned int suggestedLength(unsigned int pendingLength);
+
   virtual int locateMessage(const unsigned char *start,
                                 const unsigned char *end,
                                     unsigned int &controlLength,
