@@ -1,15 +1,15 @@
 /**************************************************************************/
 /*                                                                        */
-/* Copyright (c) 2001, 2006 NoMachine, http://www.nomachine.com/.         */
+/* Copyright (c) 2001, 2007 NoMachine, http://www.nomachine.com/.         */
 /*                                                                        */
-/* NXPROXY, NX protocol compression and NX extensions to this software    */
+/* NXCOMP, NX protocol compression and NX extensions to this software     */
 /* are copyright of NoMachine. Redistribution and use of the present      */
 /* software is allowed according to terms specified in the file LICENSE   */
 /* which comes in the source distribution.                                */
 /*                                                                        */
 /* Check http://www.nomachine.com/licensing.html for applicability.       */
 /*                                                                        */
-/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
+/* NX and NoMachine are trademarks of NoMachine S.r.l.                    */
 /*                                                                        */
 /* All rights reserved.                                                   */
 /*                                                                        */
@@ -23,12 +23,6 @@
 
 #include "Keeper.h"
 
-//
-// Sleep once any ONCE entries.
-//
-
-#define ONCE  2
-
 //
 // Set the verbosity level.
 //
@@ -38,6 +32,19 @@
 #undef  TEST
 #undef  DEBUG
 
+//
+// Remove the directory if it's empty
+// since more than 30 * 24 h.
+//
+
+#define EMPTY_DIR_TIME  2592000
+
+//
+// Sleep once any ONCE entries.
+//
+
+#define ONCE  2
+
 //
 // Define this to trace messages while
 // they are allocated and deallocated.
@@ -85,12 +92,8 @@ File::File()
 // also the string pointed in the File structure is dele-
 // ted. Everything is logged, but still the memory is not
 // freed. This is less a problem on Windows, where the me-
-// mory occupation remains almost constant, another proof
-// that the problem lies in some specific "optimization"
-// of libstc++ for Linux. Additionally the problem is not
-// with the set<> container. Valgrind says that the objects
-// that are not freed are the File objects we allocate in
-// collect().
+// mory occupation remains almost constant. Obviously the
+// problem lies in the STL allocators of the GNU libstc++.
 //
 
 File::~File()
@@ -382,6 +385,7 @@ int Keeper::collect(const char *path)
     int baseSize = strlen(path);
     int fileSize = baseSize + 3 + MD5_LENGTH * 2 + 1;
 
+    int n = 0;
     int s = 0;
 
     while (((dirEntry = readdir(cacheDir)) != NULL))
@@ -390,6 +394,14 @@ int Keeper::collect(const char *path)
 
       if (signal_ != 0) break;
 
+      if (strcmp(dirEntry -> d_name, ".") == 0 ||
+              strcmp(dirEntry -> d_name, "..") == 0)
+      {
+        continue;
+      }      
+
+      n++;
+
       if (strlen(dirEntry -> d_name) == (MD5_LENGTH * 2 + 2) &&
               (strncmp(dirEntry -> d_name, "I-", 2) == 0 ||
                   strncmp(dirEntry -> d_name, "S-", 2) == 0 ||
@@ -449,16 +461,40 @@ int Keeper::collect(const char *path)
     }
 
     closedir(cacheDir);
+
+    if (n == 0)
+    {
+      time_t now = time(NULL);
+
+      if (now > 0 && stat(path, &fileStat) == 0)
+      {
+        #ifdef TEST
+        *logofs << "Keeper: Empty NX subdirectory '" << path
+                << "' unused since " << now - fileStat.st_mtime
+                << " S.\n" << logofs_flush;
+        #endif
+
+        if (now - fileStat.st_mtime > EMPTY_DIR_TIME)
+        {
+          #ifdef TEST
+          *logofs << "Keeper: Removing empty NX subdirectory '"
+                  << path << "'.\n" << logofs_flush;
+          #endif
+
+          rmdir(path);
+        }
+      }
+    }
   }
   else
   {
     #ifdef WARNING
-    *logofs << "Keeper: WARNING! Can't open NX images subdirectory '"
+    *logofs << "Keeper: WARNING! Can't open NX subdirectory '"
             << path << ". Error is " << EGET() << " '" << ESTR()
             << "'.\n" << logofs_flush;
      #endif
 
-     cerr << "Warning" << ": Can't open NX images subdirectory '"
+     cerr << "Warning" << ": Can't open NX subdirectory '"
           << path << ". Error is " << EGET() << " '" << ESTR()
           << "'.\n";
   }
