@@ -1,15 +1,15 @@
 /**************************************************************************/
 /*                                                                        */
-/* Copyright (c) 2001, 2006 NoMachine, http://www.nomachine.com/.         */
+/* Copyright (c) 2001, 2007 NoMachine, http://www.nomachine.com/.         */
 /*                                                                        */
-/* NXPROXY, NX protocol compression and NX extensions to this software    */
+/* NXCOMP, NX protocol compression and NX extensions to this software     */
 /* are copyright of NoMachine. Redistribution and use of the present      */
 /* software is allowed according to terms specified in the file LICENSE   */
 /* which comes in the source distribution.                                */
 /*                                                                        */
 /* Check http://www.nomachine.com/licensing.html for applicability.       */
 /*                                                                        */
-/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
+/* NX and NoMachine are trademarks of NoMachine S.r.l.                    */
 /*                                                                        */
 /* All rights reserved.                                                   */
 /*                                                                        */
@@ -144,8 +144,6 @@ MessageStore::MessageStore(StaticCompressor *compressor)
 
   temporary_ = NULL;
 
-  kept_ = 0;
-
   #ifdef REFERENCES
 
   references_++;
@@ -316,17 +314,19 @@ int MessageStore::parseData(Message *message, int split, const unsigned char *bu
     }
 
     //
-    // Accept anyway data beyond the limit
-    // but issue a warning.
+    // Accept anyway data beyond the
+    // expected limit.
     //
 
-    #ifdef WARNING
+    #ifdef TEST
+
     if (dataSize > (unsigned int) dataLimit)
     {
       *logofs << name() << ": WARNING! Data is " << dataSize
-              << " bytes. Ignoring the established limit!\n"
+              << " bytes. Ignoring the established limit.\n"
               << logofs_flush;
     }
+
     #endif
 
     if (dataSize != message -> data_.size())
@@ -637,21 +637,6 @@ int MessageStore::clean(T_checksum_action checksumAction)
 
       return nothing;
     }
-    else if ((*messages_)[position] -> keep_ != 0 &&
-                  (getRemoteStorageSize() <
-                       (control -> RemoteTotalStorageSize / 100 *
-                           cacheThreshold)) && (getLocalStorageSize() <
-                                (control -> LocalTotalStorageSize / 100 *
-                                    cacheThreshold)))
-    {
-      #ifdef WARNING
-      *logofs << name() << ": WARNING! Skipping removal "
-              << "of startup message.\n"
-              << logofs_flush;
-      #endif
-
-      return nothing;
-    }
 
     #ifdef DEBUG
     *logofs << name() << ": WARNING! Assuming object "
@@ -660,61 +645,9 @@ int MessageStore::clean(T_checksum_action checksumAction)
     #endif
   }
 
-  #ifdef TEST
-
-  if ((*messages_)[position] -> keep_ != 0)
-  {
-    #ifdef WARNING
-    *logofs << name() << ": WARNING! Cleanup of position "
-            << position << " is going to remove a startup "
-            << "message.\n" << logofs_flush;
-    #endif
-  }
-
-  #endif
-
   return position;
 }
 
-int MessageStore::keep(const int position)
-{
-  Message *message = (*messages_)[position];
-
-  if (message == NULL)
-  {
-    return 0;
-  }
-
-  //
-  // Never retain more than 20% of total
-  // available messages in repository.
-  //
-
-  if (kept_ < cacheSlots / 5)
-  {
-    #ifdef TEST
-    *logofs << name() << ": Set keep flag of object at "
-            << strMsTimestamp() << " with "
-            << kept_ << " objects marked.\n"
-            << logofs_flush;
-    #endif
-
-    kept_++;
-
-    return (message -> keep_ = 1);
-  }
-  else
-  {
-    #ifdef TEST
-    *logofs << name() << ": WARNING! Can't keep object with "
-            << kept_ << " messages already marked.\n"
-            << logofs_flush;
-    #endif
-
-    return 0;
-  }
-}
-
 //
 // This is the insertion method used at local side 
 // side. Cache at remote side side will be kept in
@@ -860,53 +793,23 @@ int MessageStore::findOrAdd(Message *message, T_checksum_action checksumAction,
 
   lastRated = position;
 
-  if ((*messages_)[position] != NULL)
+  if ((*messages_)[position] != NULL &&
+          (*messages_)[position] -> locks_ != 0)
   {
-    if ((*messages_)[position] -> locks_ != 0)
-    {
-      #ifdef WARNING
-      *logofs << name() << ": WARNING! Insertion at position "
-              << position << " would replace a locked message. "
-              << "Forcing channel to discard the message.\n"
-              << logofs_flush;
-      #endif
+    #ifdef WARNING
+    *logofs << name() << ": WARNING! Insertion at position "
+            << position << " would replace a locked message. "
+            << "Forcing channel to discard the message.\n"
+            << logofs_flush;
+    #endif
 
-      #ifdef TEST
-      *logofs << name() << ": Invalidating rating of object "
-              << "at position " << position << ".\n"
-              << logofs_flush;
-      #endif
+    #ifdef TEST
+    *logofs << name() << ": Invalidating rating of object "
+            << "at position " << position << ".\n"
+            << logofs_flush;
+    #endif
 
-      return (lastRated = nothing);
-    }
-    else if ((*messages_)[position] -> keep_ != 0 &&
-                  (getRemoteStorageSize() <
-                       (control -> RemoteTotalStorageSize / 100 *
-                           cacheThreshold)) && (getLocalStorageSize() <
-                                (control -> LocalTotalStorageSize / 100 *
-                                    cacheThreshold)))
-    {
-      //
-      // Skip removal of this message. This
-      // will implicitly reserve more space
-      // for startup images.
-      //
-
-      #ifdef WARNING
-      *logofs << name() << ": WARNING! Insertion at position "
-              << position << " would replace a startup message. "
-              << "Forcing channel to discard the message.\n"
-              << logofs_flush;
-      #endif
-
-      #ifdef TEST
-      *logofs << name() << ": Invalidating rating of object "
-              << "at position " << position << ".\n"
-              << logofs_flush;
-      #endif
-
-      return (lastRated = nothing);
-    }
+    return (lastRated = nothing);
   }
 
   if (checksumAction == use_checksum)
@@ -1081,7 +984,6 @@ int MessageStore::findOrAdd(Message *message, T_checksum_action checksumAction,
   message -> last_ = (getTimestamp()).tv_sec;
 
   message -> locks_ = 0;
-  message -> keep_  = 0;
 
   #ifdef DEBUG
   *logofs << name() << ": Set last hit of object at " 
@@ -1183,7 +1085,6 @@ int MessageStore::add(Message *message, const int position,
   message -> last_ = (getTimestamp()).tv_sec;
 
   message -> locks_ = 0;
-  message -> keep_  = 0;
 
   #ifdef DEBUG
   *logofs << name() << ": Set last hit of object at " 
@@ -1425,15 +1326,6 @@ int MessageStore::remove(const int position, T_checksum_action checksumAction,
 
   #if defined(TEST) || defined(INFO)
 
-  if (message -> keep_ != 0)
-  {
-    #ifdef WARNING
-    *logofs << name() << ": WARNING! Insertion at position "
-            << position << " is going to replace a startup "
-            << "message.\n" << logofs_flush;
-    #endif
-  }
-
   if (opcode() == X_PutImage || opcode() == X_NXPutPackedImage)
   {
     #ifdef WARNING
@@ -1550,21 +1442,12 @@ int MessageStore::remove(const int position, T_checksum_action checksumAction,
 
 int MessageStore::getRating(Message *message, T_rating type) const
 {
-  if (message -> locks_ != 0 || message -> keep_ != 0)
+  if (message -> locks_ != 0)
   {
     #ifdef TEST
-    if (message -> locks_ != 0)
-    {
-      *logofs << name() << ": Rate set to -1 as locks of object are "
-              << (int) message -> locks_ << ".\n"
-              << logofs_flush;
-    }
-    else if (message -> keep_ != 0)
-    {
-      *logofs << name() << ": Rate set to -1 as keep flag is "
-              << (int) message -> keep_ << ".\n"
-              << logofs_flush;
-    }
+    *logofs << name() << ": Rate set to -1 as locks of object are "
+            << (int) message -> locks_ << ".\n"
+            << logofs_flush;
     #endif
 
     return -1;
