@@ -1,15 +1,15 @@
 /**************************************************************************/
 /*                                                                        */
-/* Copyright (c) 2001, 2006 NoMachine, http://www.nomachine.com/.         */
+/* Copyright (c) 2001, 2007 NoMachine, http://www.nomachine.com/.         */
 /*                                                                        */
-/* NXPROXY, NX protocol compression and NX extensions to this software    */
+/* NXCOMP, NX protocol compression and NX extensions to this software     */
 /* are copyright of NoMachine. Redistribution and use of the present      */
 /* software is allowed according to terms specified in the file LICENSE   */
 /* which comes in the source distribution.                                */
 /*                                                                        */
 /* Check http://www.nomachine.com/licensing.html for applicability.       */
 /*                                                                        */
-/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
+/* NX and NoMachine are trademarks of NoMachine S.r.l.                    */
 /*                                                                        */
 /* All rights reserved.                                                   */
 /*                                                                        */
@@ -41,7 +41,6 @@
 #define OPCODES
 #undef  TEST
 #undef  DEBUG
-#undef  DUMP
 
 //
 // TCP port offset applied to any NX port specification.
@@ -297,13 +296,18 @@ keyboard=s\n\
 clipboard=n\n\
 streaming=n\n\
 backingstore=n\n\
-rdpcolors=n\n\
-rdpcache=n\n\
+composite=n\n\
 shmem=b\n\
 shpix=b\n\
 kbtype=s\n\
-client=s       These options are interpreted by the NX agents. They\n\
-               have no effect on the proxy.\n\
+client=s\n\
+shadow=n\n\
+shadowuid=n\n\
+shadowmode=s\n\
+defer=n\n\
+tile=s\n\
+menu=n         These options are interpreted by the NX agent. They\n\
+               are ignored by the proxy.\n\
 \n\
   Environment:\n\
 \n\
@@ -368,16 +372,16 @@ const char *GetUsageInfo()
 
 static const char CopyrightInfo[] =
 "\
-Copyright (c) 2001, 2006 NoMachine, http://www.nomachine.com/.\n\
+Copyright (c) 2001, 2007 NoMachine, http://www.nomachine.com/.\n\
 \n\
-NXPROXY, NX protocol compression and NX extensions to this software\n\
+NXCOMP, NX protocol compression and NX extensions to this software \n\
 are copyright of NoMachine. Redistribution and use of the present\n\
 software is allowed according to terms specified in the file LICENSE\n\
 which comes in the source distribution.\n\
 \n\
 Check http://www.nomachine.com/licensing.html for applicability.\n\
 \n\
-NX and NoMachine are trademarks of Medialogic S.p.A.\n\
+NX and NoMachine are trademarks of NoMachine S.r.l.\n\
 \n\
 All rights reserved.\n\
 ";
@@ -407,13 +411,50 @@ const char *GetOtherCopyrightInfo()
 int _hostBigEndian  = 0;
 int _storeBigEndian = 0;
 
+const unsigned int IntMask[33] =
+{
+  0x00000000,
+  0x00000001,
+  0x00000003,
+  0x00000007,
+  0x0000000f,
+  0x0000001f,
+  0x0000003f,
+  0x0000007f,
+  0x000000ff,
+  0x000001ff,
+  0x000003ff,
+  0x000007ff,
+  0x00000fff,
+  0x00001fff,
+  0x00003fff,
+  0x00007fff,
+  0x0000ffff,
+  0x0001ffff,
+  0x0003ffff,
+  0x0007ffff,
+  0x000fffff,
+  0x001fffff,
+  0x003fffff,
+  0x007fffff,
+  0x00ffffff,
+  0x01ffffff,
+  0x03ffffff,
+  0x07ffffff,
+  0x0fffffff,
+  0x1fffffff,
+  0x3fffffff,
+  0x7fffffff,
+  0xffffffff
+};
+
 unsigned int GetUINT(unsigned const char *buffer, int bigEndian)
 {
   //
-  // TODO: It doesn't work on SPARCs if buffer
-  // is not aligned to word boundary. We should
-  // check the CPU, not the OS as this probably
-  // applies to other processors.
+  // It doesn't work on SPARCs if the buffer
+  // is not aligned to the word boundary. We
+  // should check the CPU, not the OS as this
+  // surely applies to other architectures.
   //
 
   #ifndef __sun
@@ -457,10 +498,8 @@ unsigned int GetUINT(unsigned const char *buffer, int bigEndian)
 unsigned int GetULONG(unsigned const char *buffer, int bigEndian)
 {
   //
-  // TODO: It doesn't work on SPARCs if buffer
-  // is not aligned to word boundary. We should
-  // check the CPU, not the OS as this probably
-  // applies to other processors.
+  // It doesn't work on SPARCs if the buffer
+  // is not aligned to word the boundary.
   //
 
   #ifndef __sun
@@ -542,7 +581,7 @@ void PutULONG(unsigned int value, unsigned char *buffer, int bigEndian)
   {
     buffer += 3;
 
-    for (int i = 4; i; i--)
+    for (int i = 4; i > 0; i--)
     {
       *buffer-- = (unsigned char) (value & 0xff);
 
@@ -551,7 +590,7 @@ void PutULONG(unsigned int value, unsigned char *buffer, int bigEndian)
   }
   else
   {
-    for (int i = 4; i; i--)
+    for (int i = 4; i > 0; i--)
     {
       *buffer++ = (unsigned char) (value & 0xff);
 
@@ -757,37 +796,6 @@ const char *DumpSignal(int signal)
   }
 }
 
-const char *DumpFlush(int type)
-{
-  switch ((T_flush) type)
-  {
-    case flush_if_needed:
-    {
-      return "if needed";
-    }
-    case flush_if_any:
-    {
-      return "if any";
-    }
-    case flush_if_sync:
-    {
-      return "if sync";
-    }
-    default:
-    {
-      #ifdef PANIC
-      *logofs << "Misc: PANIC! Unknown flush type '"
-              << type << "'.\n" << logofs_flush;
-      #endif
-
-      cerr << "Error" << ": Unknown flush type '"
-           << type << "'.\n";
-
-      HandleCleanup();
-    }
-  }
-}
-
 const char *DumpPolicy(int type)
 {
   switch ((T_flush_policy) type)
@@ -817,36 +825,35 @@ const char *DumpPolicy(int type)
 
 const char *DumpAction(int type)
 {
-  switch ((T_store_action) type)
+  T_store_action action = (T_store_action) type;
+
+  if (action == IS_HIT)
   {
-    case is_added:
-    {
-      return "is_added";
-    }
-    case is_hit:
-    {
-      return "is_hit";
-    }
-    case is_discarded:
-    {
-      return "is_discarded";
-    }
-    case is_removed:
-    {
-      return "is_removed";
-    }
-    default:
-    {
-      #ifdef PANIC
-      *logofs << "Misc: PANIC! Unknown store action '"
-              << type << "'.\n" << logofs_flush;
-      #endif
+    return "is_hit";
+  }
+  else if (action == IS_ADDED)
+  {
+    return "is_added";
+  }
+  else if (action == is_discarded)
+  {
+    return "is_discarded";
+  }
+  else if (action == is_removed)
+  {
+    return "is_removed";
+  }
+  else
+  {
+    #ifdef PANIC
+    *logofs << "Misc: PANIC! Unknown store action '"
+            << type << "'.\n" << logofs_flush;
+    #endif
 
-      cerr << "Error" << ": Unknown store action '"
-           << type << "'.\n";
+    cerr << "Error" << ": Unknown store action '"
+         << type << "'.\n";
 
-      HandleCleanup();
-    }
+    HandleCleanup();
   }
 }
 
@@ -1056,21 +1063,17 @@ const char *DumpSession(int code)
 {
   switch ((T_session_mode) code)
   {
-    case session_x:
+    case session_agent:
     {
-      return "session_x";
+      return "session_agent";
     }
-    case session_rdp:
+    case session_shadow:
     {
-      return "session_rdp";
+      return "session_shadow";
     }
-    case session_rfb:
+    case session_proxy:
     {
-      return "session_rfb";
-    }
-    case session_raw:
-    {
-      return "session_raw";
+      return "session_proxy";
     }
     default:
     {
@@ -1147,7 +1150,7 @@ const char *DumpChecksum(const void *checksum)
 // if you want to log the opcode literal.
 //
 
-#if defined(DUMP) || defined(OPCODES)
+#ifdef OPCODES
 
 const char *DumpOpcode(const int &opcode)
 {
@@ -1749,21 +1752,14 @@ const char *DumpOpcode(const int &opcode)
   }
 }
 
-#else /* #if defined(DUMP) || defined(OPCODES) */
+#else /* #ifdef OPCODES */
 
 const char *DumpOpcode(const int &opcode)
 {
   return "?";
 }
 
-#endif /* #if defined(DUMP) || defined(OPCODES) */
-
-//
-// Save space by compiling these functions
-// only if DUMP explicitly defined.
-//
-
-#ifdef DUMP
+#endif /* #ifdef OPCODES */
 
 void DumpData(const unsigned char *buffer, unsigned int size)
 {
@@ -1896,5 +1892,3 @@ void DumpHexData(const unsigned char *buffer, unsigned int size)
 
   *logofs << message << logofs_flush;
 }
-
-#endif /* #ifdef DUMP */
