@@ -1,15 +1,15 @@
 /**************************************************************************/
 /*                                                                        */
-/* Copyright (c) 2001, 2006 NoMachine, http://www.nomachine.com/.         */
+/* Copyright (c) 2001, 2007 NoMachine, http://www.nomachine.com/.         */
 /*                                                                        */
-/* NXPROXY, NX protocol compression and NX extensions to this software    */
+/* NXCOMP, NX protocol compression and NX extensions to this software     */
 /* are copyright of NoMachine. Redistribution and use of the present      */
 /* software is allowed according to terms specified in the file LICENSE   */
 /* which comes in the source distribution.                                */
 /*                                                                        */
 /* Check http://www.nomachine.com/licensing.html for applicability.       */
 /*                                                                        */
-/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
+/* NX and NoMachine are trademarks of NoMachine S.r.l.                    */
 /*                                                                        */
 /* All rights reserved.                                                   */
 /*                                                                        */
@@ -150,13 +150,12 @@ void ResetTimer();
 void HandleAlert(int code, int local);
 
 //
-// Run the callback registered by the agent.
+// Run the callback registered by the proxy
+// or the agent.
 //
 
-void FlushCallback();
-void CongestionCallback(int type);
-void SynchronizationCallback(int type);
 void KeeperCallback();
+void FlushCallback(int length);
 
 //
 // Return the string literal corresponding
@@ -164,7 +163,6 @@ void KeeperCallback();
 //
 
 const char *DumpSignal(int signal);
-const char *DumpFlush(int type);
 const char *DumpPolicy(int type);
 const char *DumpControl(int code);
 const char *DumpSession(int code);
@@ -226,6 +224,8 @@ inline int storeBigEndian()
   return _storeBigEndian;
 }
 
+extern const unsigned int IntMask[33];
+
 unsigned int GetUINT(unsigned const char *buffer, int bigEndian);
 unsigned int GetULONG(unsigned const char *buffer, int bigEndian);
 void PutUINT(unsigned int value, unsigned char *buffer, int bigEndian);
