@@ -1,15 +1,15 @@
 /**************************************************************************/
 /*                                                                        */
-/* Copyright (c) 2001, 2006 NoMachine, http://www.nomachine.com/.         */
+/* Copyright (c) 2001, 2007 NoMachine, http://www.nomachine.com/.         */
 /*                                                                        */
-/* NXPROXY, NX protocol compression and NX extensions to this software    */
+/* NXCOMP, NX protocol compression and NX extensions to this software     */
 /* are copyright of NoMachine. Redistribution and use of the present      */
 /* software is allowed according to terms specified in the file LICENSE   */
 /* which comes in the source distribution.                                */
 /*                                                                        */
 /* Check http://www.nomachine.com/licensing.html for applicability.       */
 /*                                                                        */
-/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
+/* NX and NoMachine are trademarks of NoMachine S.r.l.                    */
 /*                                                                        */
 /* All rights reserved.                                                   */
 /*                                                                        */
@@ -94,11 +94,12 @@ static char *tmpBuf;
 static int  tmpBufSize = 0;
 static int  streamPos;
 
-int UnpackPng(T_geometry *geometry, unsigned char method,
-                   unsigned char *srcData, int srcSize, int dstBpp,
-                       int dstWidth, int dstHeight, unsigned char *dstData,
-                           int dstSize, int byteOrder)
+int UnpackPng(T_geometry *geometry, unsigned char method, unsigned char *srcData,
+                  int srcSize, int dstBpp, int dstWidth, int dstHeight,
+                      unsigned char *dstData, int dstSize)
 {
+  int byteOrder = geometry -> image_byte_order;
+
   //
   // Check if data is coming from a failed unsplit.
   //
@@ -585,7 +586,7 @@ int DecompressPng24(unsigned char *compressedData, int compressedLen,
       // Follow the server byte order when arranging data.
       //
 
-      if (byteOrder == 0)
+      if (byteOrder == LSBFirst)
       {
         pixelPtr[0] = tmpBuf[dx * 3];
         pixelPtr[1] = tmpBuf[dx * 3 + 1];
@@ -736,7 +737,7 @@ int DecompressPng32(unsigned char *compressedData, int compressedLen,
       // Follow the server byte order when arranging data.
       //
 
-      if (byteOrder == 0)
+      if (byteOrder == LSBFirst)
       {
         for (i = 0; i < 4; i++)
         {
