@@ -1,15 +1,15 @@
 /**************************************************************************/
 /*                                                                        */
-/* Copyright (c) 2001, 2006 NoMachine, http://www.nomachine.com/.         */
+/* Copyright (c) 2001, 2007 NoMachine, http://www.nomachine.com/.         */
 /*                                                                        */
-/* NXPROXY, NX protocol compression and NX extensions to this software    */
+/* NXCOMP, NX protocol compression and NX extensions to this software     */
 /* are copyright of NoMachine. Redistribution and use of the present      */
 /* software is allowed according to terms specified in the file LICENSE   */
 /* which comes in the source distribution.                                */
 /*                                                                        */
 /* Check http://www.nomachine.com/licensing.html for applicability.       */
 /*                                                                        */
-/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
+/* NX and NoMachine are trademarks of NoMachine S.r.l.                    */
 /*                                                                        */
 /* All rights reserved.                                                   */
 /*                                                                        */
@@ -27,11 +27,41 @@
 
 #define WARNING
 #define PANIC
-#undef  OPCODES
 #undef  TEST
 #undef  DEBUG
 #undef  DUMP
 
+unsigned int ProxyReadBuffer::suggestedLength(unsigned int pendingLength)
+{
+  //
+  // Always read all the data that
+  // is available.
+  //
+
+  int readable = transport_ -> readable();
+
+  unsigned int readLength = (readable == -1 ? 0 : (unsigned int) readable);
+
+  if (readLength < pendingLength)
+  {
+    readLength = pendingLength;
+  }
+
+  //
+  // Even if the readable data is not
+  // enough to make a complete message,
+  // resize the buffer to accomodate
+  // it all.
+  //
+
+  if (pendingLength < remaining_)
+  {
+    readLength = remaining_;
+  }
+
+  return readLength;
+}  
+
 int ProxyReadBuffer::locateMessage(const unsigned char *start,
                                        const unsigned char *end,
                                            unsigned int &controlLength,
@@ -47,8 +77,7 @@ int ProxyReadBuffer::locateMessage(const unsigned char *start,
   #ifdef TEST
   *logofs << "ProxyReadBuffer: Locating message for FD#"
           << transport_ -> fd() << " with " << end - start
-          << " bytes and remaining " << remaining_ << ".\n"
-          << logofs_flush;
+          << " bytes.\n" << logofs_flush;
   #endif
 
   //
@@ -72,14 +101,14 @@ int ProxyReadBuffer::locateMessage(const unsigned char *start,
   {
     if (nextSrc >= end)
     {
+      remaining_ = 1;
+
       #ifdef TEST
       *logofs << "ProxyReadBuffer: No message was located "
               << "with remaining " << remaining_ << ".\n"
               << logofs_flush;
       #endif
 
-      remaining_ = 1;
-
       return 0;
     }
 
@@ -109,7 +138,20 @@ int ProxyReadBuffer::locateMessage(const unsigned char *start,
 
   if (start + totalLength > end)
   {
-    remaining_ = totalLength - (end - start);
+    //
+    // When having to decompress a ZLIB stream,
+    // a single byte can be enough to complete
+    // the frame.
+    //
+
+    if (control -> RemoteStreamCompression == 0)
+    {
+      remaining_ = totalLength - (end - start);
+    }
+    else
+    {
+      remaining_ = 1;
+    }
 
     #ifdef TEST
     *logofs << "ProxyReadBuffer: No message was located "
@@ -130,7 +172,7 @@ int ProxyReadBuffer::locateMessage(const unsigned char *start,
     *logofs << " on proxy FD#" << transport_ -> fd() << ".\n" << logofs_flush;
     #endif
 
-    #if defined(TEST) || defined(OPCODES)
+    #if defined(TEST) || defined(INFO)
     *logofs << "ProxyReadBuffer: Produced plain input for " << dataLength 
             << "+" << trailerLength << "+" << controlLength << " bytes out of " 
             << totalLength << " bytes.\n" << logofs_flush;
