@@ -1,15 +1,15 @@
 /**************************************************************************/
 /*                                                                        */
-/* Copyright (c) 2001, 2006 NoMachine, http://www.nomachine.com/.         */
+/* Copyright (c) 2001, 2007 NoMachine, http://www.nomachine.com/.         */
 /*                                                                        */
-/* NXPROXY, NX protocol compression and NX extensions to this software    */
+/* NXCOMP, NX protocol compression and NX extensions to this software     */
 /* are copyright of NoMachine. Redistribution and use of the present      */
 /* software is allowed according to terms specified in the file LICENSE   */
 /* which comes in the source distribution.                                */
 /*                                                                        */
 /* Check http://www.nomachine.com/licensing.html for applicability.       */
 /*                                                                        */
-/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
+/* NX and NoMachine are trademarks of NoMachine S.r.l.                    */
 /*                                                                        */
 /* All rights reserved.                                                   */
 /*                                                                        */
@@ -26,7 +26,6 @@
 #include "ServerReadBuffer.h"
 
 #include "Unpack.h"
-#include "Tight.h"
 
 //
 // Set the verbosity level.
@@ -65,7 +64,8 @@ class ServerChannel : public Channel
 
   virtual ~ServerChannel();
 
-  virtual int handleRead(EncodeBuffer &encodeBuffer);
+  virtual int handleRead(EncodeBuffer &encodeBuffer, const unsigned char *message,
+                             unsigned int length);
 
   virtual int handleWrite(const unsigned char *message, unsigned int length);
 
@@ -111,11 +111,8 @@ class ServerChannel : public Channel
 
   virtual int handleFinish();
 
-  virtual int handleNotify(T_notification_type type)
-  {
-    return 0;
-  }
-    
+  virtual int handleAsyncEvents();
+
   virtual int needSplit() const
   {
     return 0;
@@ -126,35 +123,9 @@ class ServerChannel : public Channel
     return (lastMotion_[0] != '\0');
   }
 
-  virtual int needLimit() const
-  {
-    return (transport_ -> length() >=
-                control -> TransportXBufferLimit);
-  }
-
-  virtual int getLimit() const
-  {
-    return (control -> TransportXBufferLimit);
-  }
-
   virtual T_channel_type getType() const
   {
-    return channel_x;
-  }
-
-  //
-  // Used to implement synchronization
-  // between proxies.
-  //
-
-  virtual int getClientSequence() const
-  {
-    return clientSequence_;
-  }
-
-  virtual int getServerSequence() const
-  {
-    return serverSequence_;
+    return channel_x11;
   }
 
   int setBigEndian(int flag);
@@ -345,11 +316,11 @@ class ServerChannel : public Channel
   {
     if (control -> isProtoStep7() == 0)
     {
-      //#if defined(TEST) || defined(SPLIT)
+      #if defined(TEST) || defined(SPLIT)
       *logofs << "handleSplitEnable: WARNING! Disabling load "
               << "and save with an old proxy version.\n"
               << logofs_flush;
-      //#endif
+      #endif
 
       splitState_.save = 0;
       splitState_.load = 0;
@@ -411,41 +382,6 @@ class ServerChannel : public Channel
 
   T_unpack_state *unpackState_[256];
 
-  //
-  // Handle creation of the special
-  // unpacker for tight images.
-  //
-
-  void handleTightStateAlloc()
-  {
-    if (tightState_ == NULL)
-    {
-      tightState_ = new Tight();
-    }
-  }
-
-  void handleTightStateRemove()
-  {
-    delete tightState_;
-
-    tightState_ = NULL;
-  }
-
-  //
-  // Check if it's time to give up reading
-  // from the channel.
-  //
-
-  int handleProxyEvents();
-
-  //
-  // Check the X socket and eventually read
-  // asynchronous the events while decoding
-  // the proxy messages.
-  //
-
-  int handleChannelEvents();
-
   //
   // Own read buffer. It is able to identify
   // full messages read from X descriptor.
@@ -492,12 +428,6 @@ class ServerChannel : public Channel
   unsigned int enableGraphicsExpose_;
   unsigned int enableNoExpose_;
 
-  //
-  // Unpack tight images.
-  //
-
-  Tight *tightState_;
-
   //
   // Used in initialization and handling
   // of MIT-SHM shared memory put images.
