@@ -1,15 +1,15 @@
 /**************************************************************************/
 /*                                                                        */
-/* Copyright (c) 2001, 2006 NoMachine, http://www.nomachine.com/.         */
+/* Copyright (c) 2001, 2007 NoMachine, http://www.nomachine.com/.         */
 /*                                                                        */
-/* NXPROXY, NX protocol compression and NX extensions to this software    */
+/* NXCOMP, NX protocol compression and NX extensions to this software     */
 /* are copyright of NoMachine. Redistribution and use of the present      */
 /* software is allowed according to terms specified in the file LICENSE   */
 /* which comes in the source distribution.                                */
 /*                                                                        */
 /* Check http://www.nomachine.com/licensing.html for applicability.       */
 /*                                                                        */
-/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
+/* NX and NoMachine are trademarks of NoMachine S.r.l.                    */
 /*                                                                        */
 /* All rights reserved.                                                   */
 /*                                                                        */
@@ -243,7 +243,7 @@ Split *SplitStore::add(MessageStore *store, int resource, T_split_mode mode,
 
   split -> d_size_ = size - split -> i_size_;
 
-  if (action == is_added || action == is_discarded)
+  if (action == IS_ADDED || action == is_discarded)
   {
     //
     // If the message was added to message
@@ -262,7 +262,7 @@ Split *SplitStore::add(MessageStore *store, int resource, T_split_mode mode,
     // side until it is recomposed.
     //
 
-    if (action == is_added)
+    if (action == IS_ADDED)
     {
       split -> store_ -> lock(split -> position_);
 
@@ -353,7 +353,7 @@ Split *SplitStore::add(MessageStore *store, int resource, int position,
 
   split -> d_size_ = size - split -> i_size_;
 
-  if (action == is_added || action == is_discarded)
+  if (action == IS_ADDED || action == is_discarded)
   {
     //
     // The unpack procedure will check if the
@@ -376,7 +376,7 @@ Split *SplitStore::add(MessageStore *store, int resource, int position,
     // is recomposed.
     //
 
-    if (action == is_added)
+    if (action == IS_ADDED)
     {
       split -> store_ -> lock(split -> position_);
 
@@ -1251,7 +1251,7 @@ int SplitStore::save(Split *split)
   // old mask.
   //
 
-  mode_t fileMode = umask(0077);
+  mode_t fileMode;
 
   //
   // Check if the file already exists. We try to
@@ -1275,6 +1275,8 @@ int SplitStore::save(Split *split)
     goto SplitStoreSaveError;
   }
 
+  fileMode = umask(0077);
+
   fileStream = new ofstream(fileName, ios::out | ios::binary);
 
   umask(fileMode);
@@ -1744,7 +1746,7 @@ int CommitStore::expand(Split *split, unsigned char *buffer, const int size)
 
 int CommitStore::update(Split *split)
 {
-  if (split -> action_ != is_added)
+  if (split -> action_ != IS_ADDED)
   {
     return 0;
   }
@@ -1786,7 +1788,7 @@ int CommitStore::update(Split *split)
   // or save it on disk at shutdown.
   //
 
-  if (split -> action_ == is_added)
+  if (split -> action_ == IS_ADDED)
   {
     split -> store_ -> unlock(split -> position_);
 
