@@ -1,15 +1,15 @@
 /**************************************************************************/
 /*                                                                        */
-/* Copyright (c) 2001, 2006 NoMachine, http://www.nomachine.com/.         */
+/* Copyright (c) 2001, 2007 NoMachine, http://www.nomachine.com/.         */
 /*                                                                        */
-/* NXPROXY, NX protocol compression and NX extensions to this software    */
+/* NXCOMP, NX protocol compression and NX extensions to this software     */
 /* are copyright of NoMachine. Redistribution and use of the present      */
 /* software is allowed according to terms specified in the file LICENSE   */
 /* which comes in the source distribution.                                */
 /*                                                                        */
 /* Check http://www.nomachine.com/licensing.html for applicability.       */
 /*                                                                        */
-/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
+/* NX and NoMachine are trademarks of NoMachine S.r.l.                    */
 /*                                                                        */
 /* All rights reserved.                                                   */
 /*                                                                        */
@@ -199,6 +199,20 @@ Statistics::Statistics(Proxy *proxy) : proxy_(proxy)
   proxyData_.dataCount_     = 0;
 
   proxyData_.streamRatio_ = 1;
+
+  startShortFrameTs_ = getTimestamp();
+  startLongFrameTs_  = getTimestamp();
+  startFrameTs_      = getTimestamp();
+
+  bytesInShortFrame_ = 0;
+  bytesInLongFrame_  = 0;
+
+  bitrateInShortFrame_ = 0;
+  bitrateInLongFrame_  = 0;
+
+  topBitrate_ = 0;
+
+  congestionInFrame_ = 0;
 }
 
 Statistics::~Statistics()
@@ -282,6 +296,227 @@ int Statistics::resetPartialStats()
   return 1;
 }
 
+void Statistics::addCompressedBytes(unsigned int bytesIn, unsigned int bytesOut)
+{
+  transportPartial_.compressedBytesIn_ += bytesIn;
+  transportTotal_.compressedBytesIn_ += bytesIn;
+
+  transportPartial_.compressedBytesOut_ += bytesOut;
+  transportTotal_.compressedBytesOut_ += bytesOut;
+
+  double ratio = bytesIn / bytesOut;
+
+  if (ratio < 1)
+  {
+    ratio = 1;
+  }
+
+  #if defined(TEST) || defined(TOKEN)
+  *logofs << "Statistics: TOKEN! Old ratio was "
+          << proxyData_.streamRatio_ << " current is "
+          << (double) ratio << " new ratio is " << (double)
+             ((proxyData_.streamRatio_ * 2) + ratio) / 3 << ".\n"
+          << logofs_flush;
+  #endif
+
+  proxyData_.streamRatio_ = ((proxyData_.streamRatio_ * 2) + ratio) / 3;
+
+  #if defined(TEST) || defined(TOKEN)
+  *logofs << "Statistics: TOKEN! Updated compressed bytes "
+          << "with " << bytesIn << " in " << bytesOut << " out "
+          << "and ratio " << (double) proxyData_.streamRatio_
+          << ".\n" << logofs_flush;
+  #endif
+}
+
+//
+// Recalculate the current bitrate. The bytes written
+// are accounted at the time the transport actually
+// writes the data to the network, not at the time it
+// receives the data from the upper layers. The reason
+// is that data can be compressed by the stream com-
+// pressor, so we can become aware of the new bitrate
+// only afer having flushed the ZLIB stream. This also
+// means that, to have a reliable estimate, we need to
+// flush the link often.
+//
+
+void Statistics::updateBitrate(int bytes)
+{
+  T_timestamp thisFrameTs = getNewTimestamp();
+
+  int diffFramesInMs = diffTimestamp(startFrameTs_, thisFrameTs);
+
+  #ifdef DEBUG
+  *logofs << "Statistics: Difference since previous timestamp is "
+          << diffFramesInMs << " Ms.\n" << logofs_flush;
+  #endif
+
+  if (diffFramesInMs > 0)
+  {
+    #ifdef DEBUG
+    *logofs << "Statistics: Removing " << diffFramesInMs
+            << " Ms in short and long time frame.\n"
+            << logofs_flush;
+    #endif
+
+    int shortBytesToRemove = (int) (((double) bytesInShortFrame_ * (double) diffFramesInMs) /
+                                       (double) control -> ShortBitrateTimeFrame);
+
+    int longBytesToRemove = (int) (((double) bytesInLongFrame_ * (double) diffFramesInMs) /
+                                      (double) control -> LongBitrateTimeFrame);
+
+    #ifdef DEBUG
+    *logofs << "Statistics: Removing " << shortBytesToRemove
+            << " bytes from " << bytesInShortFrame_
+            << " in the short frame.\n" << logofs_flush;
+    #endif
+
+    bytesInShortFrame_ -= shortBytesToRemove;
+
+    if (bytesInShortFrame_ < 0)
+    {
+      #ifdef TEST
+      *logofs << "Statistics: Bytes in short frame are "
+              << bytesInShortFrame_ << ". Set to 0.\n"
+              << logofs_flush;
+      #endif
+
+      bytesInShortFrame_ = 0;
+    }
+
+    #ifdef DEBUG
+    *logofs << "Statistics: Removing " << longBytesToRemove
+            << " bytes from " << bytesInLongFrame_
+            << " in the long frame.\n" << logofs_flush;
+    #endif
+
+    bytesInLongFrame_ -= longBytesToRemove;
+
+    if (bytesInLongFrame_ < 0)
+    {
+      #ifdef TEST
+      *logofs << "Statistics: Bytes in long frame are "
+              << bytesInLongFrame_ << ". Set to 0.\n"
+              << logofs_flush;
+      #endif
+
+      bytesInLongFrame_ = 0;
+    }
+
+    int diffStartInMs;
+
+    diffStartInMs = diffTimestamp(thisFrameTs, startShortFrameTs_);
+
+    if (diffStartInMs > control -> ShortBitrateTimeFrame)
+    {
+      addMsToTimestamp(startShortFrameTs_, diffStartInMs);
+    }
+
+    diffStartInMs = diffTimestamp(thisFrameTs, startLongFrameTs_);
+
+    if (diffStartInMs > control -> LongBitrateTimeFrame)
+    {
+      addMsToTimestamp(startLongFrameTs_, diffStartInMs);
+    }
+
+    startFrameTs_ = thisFrameTs;
+  }
+
+  #ifdef DEBUG
+  *logofs << "Statistics: Adding " << bytes << " bytes to "
+          << bytesInShortFrame_ << " in the short frame.\n"
+          << logofs_flush;
+  #endif
+
+  bytesInShortFrame_ = bytesInShortFrame_ + bytes;
+
+  #ifdef DEBUG
+  *logofs << "Statistics: Adding " << bytes << " bytes to "
+          << bytesInLongFrame_ << " in the long frame.\n"
+          << logofs_flush;
+  #endif
+
+  bytesInLongFrame_ = bytesInLongFrame_ + bytes;
+
+  bitrateInShortFrame_ = (int) ((double) bytesInShortFrame_ /
+                                   ((double) control -> ShortBitrateTimeFrame / 1000));
+
+  bitrateInLongFrame_ = (int) ((double) bytesInLongFrame_ /
+                                   ((double) control -> LongBitrateTimeFrame / 1000));
+
+  if (bitrateInShortFrame_ > topBitrate_)
+  {
+    topBitrate_ = bitrateInShortFrame_;
+  }
+
+  #ifdef TEST
+  *logofs << "Statistics: Current bitrate is short " << bitrateInShortFrame_
+          << " long " << bitrateInLongFrame_ << " top " << topBitrate_
+          << ".\n" << logofs_flush;
+  #endif
+}
+
+void Statistics::updateCongestion(int remaining, int limit)
+{
+  #ifdef TEST
+  *logofs << "Statistics: Updating the congestion "
+          << "counters at " << strMsTimestamp()
+          << ".\n" << logofs_flush;
+  #endif
+
+  double current = remaining;
+
+  if (current < 0)
+  {
+    current = 0;
+  }
+
+  current = 9 * (limit - current) / limit;
+
+  #ifdef TEST
+  *logofs << "Statistics: Current congestion is "
+          << current << " with " << limit << " tokens "
+          << "and " << remaining << " remaining.\n"
+          << logofs_flush;
+  #endif
+
+  //
+  // If the current congestion counter is greater
+  // than the previous, take the current value,
+  // otherwise ramp down the value by calculating
+  // the average of the last 8 updates.
+  //
+
+  #ifdef TEST
+  *logofs << "Statistics: Old congestion was "
+          << congestionInFrame_;
+  #endif
+
+  if (current >= congestionInFrame_)
+  {
+    congestionInFrame_ = current;
+  }
+  else
+  {
+    congestionInFrame_ = ((congestionInFrame_ * 7) + current) / 8;
+  }
+
+  #ifdef TEST
+  *logofs << " new congestion is "
+          << ((congestionInFrame_ * 7) + current) / 8
+          << ".\n" << logofs_flush;
+  #endif
+
+  //
+  // Call the function with 0 bytes flushed
+  // so the agent can update its congestion
+  // counter.
+  //
+
+  FlushCallback(0);
+}
+
 int Statistics::getClientCacheStats(int type, char *&buffer)
 {
   if (type != PARTIAL_STATS && type != TOTAL_STATS)
@@ -1709,13 +1944,13 @@ int Statistics::getBitrateStats(int type, char *&buffer)
   char format[FORMAT_LENGTH];
 
   sprintf(format, "      %.0f B/s average, %d B/s %ds, %d B/s %ds, %d B/s maximum.\n\n",
-              total, control -> getBitrateInShortFrame(), control -> ShortBitrateTimeFrame / 1000,
-                  control -> getBitrateInLongFrame(), control -> LongBitrateTimeFrame / 1000,
-                      control -> getTopBitrate());
+              total, getBitrateInShortFrame(), control -> ShortBitrateTimeFrame / 1000,
+                  getBitrateInLongFrame(), control -> LongBitrateTimeFrame / 1000,
+                      getTopBitrate());
 
   strcat(buffer, format);
 
-  control -> resetTopBitrate();
+  resetTopBitrate();
 
   return 1;
 }
