@@ -1,15 +1,15 @@
 /**************************************************************************/
 /*                                                                        */
-/* Copyright (c) 2001, 2006 NoMachine, http://www.nomachine.com/.         */
+/* Copyright (c) 2001, 2007 NoMachine, http://www.nomachine.com/.         */
 /*                                                                        */
-/* NXPROXY, NX protocol compression and NX extensions to this software    */
+/* NXCOMP, NX protocol compression and NX extensions to this software     */
 /* are copyright of NoMachine. Redistribution and use of the present      */
 /* software is allowed according to terms specified in the file LICENSE   */
 /* which comes in the source distribution.                                */
 /*                                                                        */
 /* Check http://www.nomachine.com/licensing.html for applicability.       */
 /*                                                                        */
-/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
+/* NX and NoMachine are trademarks of NoMachine S.r.l.                    */
 /*                                                                        */
 /* All rights reserved.                                                   */
 /*                                                                        */
@@ -58,6 +58,13 @@ class Proxy;
 #undef  TEST
 #undef  DEBUG
 
+//
+// Log the operations related to updating
+// the control tokens.
+//
+
+#undef  TOKEN
+
 class Statistics
 {
   public:
@@ -113,12 +120,22 @@ class Statistics
     transportTotal_.proxyBytesIn_ += numBytes;
   }
 
+  double getBytesIn()
+  {
+    return transportTotal_.proxyBytesIn_;
+  }
+
   void addBytesOut(unsigned int numBytes)
   {
     transportPartial_.proxyBytesOut_ += numBytes;
     transportTotal_.proxyBytesOut_ += numBytes;
   }
 
+  double getBytesOut()
+  {
+    return transportTotal_.proxyBytesOut_;
+  }
+
   void addFrameIn()
   {
     transportPartial_.proxyFramesIn_++;
@@ -156,38 +173,7 @@ class Statistics
     #endif
   }
 
-  void addCompressedBytes(unsigned int bytesIn, unsigned int bytesOut)
-  {
-    transportPartial_.compressedBytesIn_ += bytesIn;
-    transportTotal_.compressedBytesIn_ += bytesIn;
-
-    transportPartial_.compressedBytesOut_ += bytesOut;
-    transportTotal_.compressedBytesOut_ += bytesOut;
-
-    double ratio = bytesIn / bytesOut;
-
-    if (ratio < 1)
-    {
-      ratio = 1;
-    }
-
-    #ifdef TEST
-    *logofs << "Statistics: TOKEN! Old ratio was "
-            << proxyData_.streamRatio_ << " current is "
-            << (double) ratio << " new ratio is " << (double)
-               (proxyData_.streamRatio_ + ratio) / 2 << ".\n"
-            << logofs_flush;
-    #endif
-
-    proxyData_.streamRatio_ = ((proxyData_.streamRatio_ * 3) + ratio) / 4;
-
-    #ifdef TEST
-    *logofs << "Statistics: TOKEN! Updated compressed bytes "
-            << "with " << bytesIn << " in " << bytesOut << " out "
-            << "and ratio " << (double) proxyData_.streamRatio_
-            << ".\n" << logofs_flush;
-    #endif
-  }
+  void addCompressedBytes(unsigned int bytesIn, unsigned int bytesOut);
 
   void addDecompressedBytes(unsigned int bytesIn, unsigned int bytesOut)
   {
@@ -407,33 +393,58 @@ class Statistics
   }
 
   //
-  // Add the recorded protocol data to the
-  // proxy token counters.
+  // Add the recorded protocol data to the proxy
+  // token counters. We want to send bpth the token
+  // request message and the data payload using a
+  // single system write, so we must guess how many
+  // output bytes we will generate.
   //
 
   void updateControlToken(int &count)
   {
     //
-    // Bits are updated as they are accumulated
-    // to avoid summing up all the opcodes.
+    // Total is the total number of protocol bytes
+    // generated so far. We have saved the number
+    // of bytes generated the last time the function
+    // was called so we can calculate the difference.
+    //
+    // The number of protocol bits is updated as soon
+    // as new bits are accumulated, to avoid summing
+    // up all the opcodes in this routine. We add a
+    // byte to the control bytes difference to account
+    // for the framing bits that are very likely to
+    // be added to the payload.
     //
 
     double total = proxyData_.protocolCount_ / 8;
 
-    #ifdef TEST 
+    double diff = total - proxyData_.controlCount_ + 1;
+
+    #if defined(TEST) || defined(TOKEN)
+    *logofs << "Statistics: TOKEN! Protocol bytes are "
+            << total  << " control bytes are "
+            << proxyData_.controlCount_ << " difference is "
+            << diff << ".\n" << logofs_flush;
+    #endif
+
+    count += (int) (diff / proxyData_.streamRatio_);
+
+    #if defined(TEST) || defined(TOKEN)
     *logofs << "Statistics: TOKEN! Adding "
-            << (int) ((total - proxyData_.controlCount_) /
-               (double) proxyData_.streamRatio_) << " bytes to "
-            << "the control token with " << (int) (total -
-               proxyData_.controlCount_) << " in and ratio "
-            << (double) proxyData_.streamRatio_ << ".\n"
+            << (int) (diff / proxyData_.streamRatio_)
+            << " bytes to the control token with ratio "
+            << proxyData_.streamRatio_ << ".\n"
             << logofs_flush;
     #endif
 
-    count += (int) ((total - proxyData_.controlCount_) /
-                        proxyData_.streamRatio_);
-
     proxyData_.controlCount_ = total;
+
+    #if defined(TEST) || defined(TOKEN)
+    *logofs << "Statistics: TOKEN! New control token has "
+            << count << " bytes with total control bytes "
+            << proxyData_.controlCount_ << ".\n"
+            << logofs_flush;
+    #endif
   }
 
   void updateSplitToken(int &count)
@@ -441,20 +452,33 @@ class Statistics
     double total = (protocolTotal_.requestBitsOut_[X_NXSplitData] +
                         protocolTotal_.eventBitsOut_[X_NXSplitEvent]) / 8;
 
-    #ifdef TEST
+    double diff = total - proxyData_.splitCount_;
+
+    #if defined(TEST) || defined(TOKEN)
+    *logofs << "Statistics: TOKEN! Protocol bytes are "
+            << total  << " split bytes are "
+            << proxyData_.splitCount_ << " difference is "
+            << diff << ".\n" << logofs_flush;
+    #endif
+
+    count += (int) (diff / proxyData_.streamRatio_);
+
+    #if defined(TEST) || defined(TOKEN)
     *logofs << "Statistics: TOKEN! Adding "
-            << (int) ((total - proxyData_.splitCount_) /
-               (double) proxyData_.streamRatio_) << " bytes to "
-            << "the split token with " << (int) (total -
-               proxyData_.splitCount_) << " in and ratio "
-            << (double) proxyData_.streamRatio_ << ".\n"
+            << (int) (diff / proxyData_.streamRatio_)
+            << " bytes to the split token with ratio "
+            << proxyData_.streamRatio_ << ".\n"
             << logofs_flush;
     #endif
 
-    count += (int) ((total - proxyData_.splitCount_) /
-                        proxyData_.streamRatio_);
-
     proxyData_.splitCount_ = total;
+
+    #if defined(TEST) || defined(TOKEN)
+    *logofs << "Statistics: TOKEN! New split token has "
+            << count << " bytes with total split bytes "
+            << proxyData_.splitCount_ << ".\n"
+            << logofs_flush;
+    #endif
   }
 
   void updateDataToken(int &count)
@@ -466,20 +490,63 @@ class Statistics
                                     protocolTotal_.fontBitsOut_ +
                                         protocolTotal_.slaveBitsOut_) / 8;
 
-    #ifdef TEST
+    double diff = total - proxyData_.dataCount_;
+
+    #if defined(TEST) || defined(TOKEN)
+    *logofs << "Statistics: TOKEN! Protocol bytes are "
+            << total  << " data bytes are "
+            << proxyData_.dataCount_ << " difference is "
+            << diff << ".\n" << logofs_flush;
+    #endif
+
+    count += (int) (diff / proxyData_.streamRatio_);
+
+    #if defined(TEST) || defined(TOKEN)
     *logofs << "Statistics: TOKEN! Adding "
-            << (int) ((total - proxyData_.dataCount_) /
-               (double) proxyData_.streamRatio_) << " bytes to "
-            << "the data token with " << (int) (total -
-               proxyData_.dataCount_) << " in and ratio "
-            << (double) proxyData_.streamRatio_ << ".\n"
+            << (int) (diff / proxyData_.streamRatio_)
+            << " bytes to the data token with ratio "
+            << proxyData_.streamRatio_ << ".\n"
             << logofs_flush;
     #endif
 
-    count += (int) ((total - proxyData_.dataCount_) /
-                        proxyData_.streamRatio_);
-
     proxyData_.dataCount_ = total;
+
+    #if defined(TEST) || defined(TOKEN)
+    *logofs << "Statistics: TOKEN! New data token has "
+            << count << " bytes with total data bytes "
+            << proxyData_.dataCount_ << ".\n"
+            << logofs_flush;
+    #endif
+  }
+
+  //
+  // Update the current bitrate.
+  //
+
+  void updateBitrate(int bytes);
+
+  //
+  // Return the current bitrate.
+  //
+
+  int getBitrateInShortFrame()
+  {
+    return bitrateInShortFrame_;
+  }
+
+  int getBitrateInLongFrame()
+  {
+    return bitrateInLongFrame_;
+  }
+
+  int getTopBitrate()
+  {
+    return topBitrate_;
+  }
+
+  void resetTopBitrate()
+  {
+    topBitrate_ = 0;
   }
 
   double getStreamRatio()
@@ -487,6 +554,17 @@ class Statistics
     return proxyData_.streamRatio_;
   }
 
+  //
+  // Manage the congestion level.
+  //
+
+  void updateCongestion(int remaining, int limit);
+
+  double getCongestionInFrame()
+  {
+    return congestionInFrame_;
+  }
+
   //
   // Produce a dump of the statistics on
   // the provided buffer.
@@ -627,6 +705,25 @@ class Statistics
 
   T_proxyData proxyData_;
 
+  //
+  // Used to calculate the bandwidth usage
+  // of the proxy link.
+  //
+
+  T_timestamp startShortFrameTs_;
+  T_timestamp startLongFrameTs_;
+  T_timestamp startFrameTs_;
+
+  int bytesInShortFrame_;
+  int bytesInLongFrame_;
+
+  int bitrateInShortFrame_;
+  int bitrateInLongFrame_;
+
+  int topBitrate_;
+
+  double congestionInFrame_;
+
   //
   // Need the proxy pointer to print the
   // statistics related to the client and
