@@ -1,15 +1,15 @@
 /**************************************************************************/
 /*                                                                        */
-/* Copyright (c) 2001, 2006 NoMachine, http://www.nomachine.com/.         */
+/* Copyright (c) 2001, 2007 NoMachine, http://www.nomachine.com/.         */
 /*                                                                        */
-/* NXPROXY, NX protocol compression and NX extensions to this software    */
+/* NXCOMP, NX protocol compression and NX extensions to this software     */
 /* are copyright of NoMachine. Redistribution and use of the present      */
 /* software is allowed according to terms specified in the file LICENSE   */
 /* which comes in the source distribution.                                */
 /*                                                                        */
 /* Check http://www.nomachine.com/licensing.html for applicability.       */
 /*                                                                        */
-/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
+/* NX and NoMachine are trademarks of NoMachine S.r.l.                    */
 /*                                                                        */
 /* All rights reserved.                                                   */
 /*                                                                        */
@@ -180,29 +180,30 @@ typedef struct
   T_data data_;
   int    length_;
   int    start_;
-} T_buffer;
+}
+T_buffer;
 
 //
 // The message store operation that was
 // executed for the message. The channels
 // use these values to determine how to
-// handle a message once it is received
-// at the decoding side.
-//
-// TODO: Even if these values are encoded
-// based on previous cached value, it would
-// be better to rearrange the codes with
-// is_hit at position 0.
+// handle the message after it has been
+// received at the decoding side.
 //
 
 enum T_store_action
 {
-  is_added,
   is_hit,
+  is_added,
   is_discarded,
-  is_removed
+  is_removed,
+  is_added_compat = 0,
+  is_hit_compat = 1
 };
 
+#define IS_HIT   (control -> isProtoStep8() == 1 ? is_hit   : is_hit_compat)
+#define IS_ADDED (control -> isProtoStep8() == 1 ? is_added : is_added_compat)
+
 enum T_checksum_action
 {
   use_checksum,
@@ -227,7 +228,8 @@ enum T_rating
 };
 
 //
-// How to handle writes to trasport's socket.
+// How to handle the writes to the X
+// and proxy connections.
 //
 
 enum T_write
@@ -236,15 +238,10 @@ enum T_write
   write_delayed
 };
 
-//
-// How to handle flushes to the X connections.
-//
-
 enum T_flush
 {
   flush_if_needed,
-  flush_if_any,
-  flush_if_sync
+  flush_if_any
 };
 
 //
